import logging

from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import PermissionDenied
from django.db import models
from django.db.models import F, OuterRef, Subquery
from django.forms import Form
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse_lazy
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.generic import DeleteView, DetailView, TemplateView
from django.views.generic.edit import FormMixin
from django_filters.views import FilterView

from rdmo.accounts.utils import is_site_manager
from rdmo.core.plugins import get_plugin, get_plugins
from rdmo.core.views import ObjectPermissionMixin, RedirectViewMixin
from rdmo.questions.models import Catalog
from rdmo.questions.utils import get_widgets

from ..filters import ProjectFilter
from ..models import Integration, Invite, Membership, Project, Value

logger = logging.getLogger(__name__)


class ProjectsView(LoginRequiredMixin, FilterView):
    template_name = 'projects/projects.html'
    context_object_name = 'projects'
    paginate_by = 20
    filterset_class = ProjectFilter

    def get_queryset(self):
        # prepare projects queryset for this user
        queryset = Project.objects.filter(user=self.request.user)
        for instance in queryset:
            queryset |= instance.get_descendants()
        queryset = queryset.distinct()

        # prepare subquery for role
        membership_subquery = models.Subquery(
            Membership.objects.filter(project=models.OuterRef('pk'), user=self.request.user).values('role')
        )
        queryset = queryset.annotate(role=membership_subquery)

        # prepare subquery for last_changed
        last_changed_subquery = models.Subquery(
            Value.objects.filter(project=models.OuterRef('pk')).order_by('-updated').values('updated')[:1]
        )
        queryset = queryset.annotate(last_changed=models.functions.Greatest('updated', last_changed_subquery))

        # order by last changed
        queryset = queryset.order_by('-last_changed')

        return queryset

    def get_context_data(self, **kwargs):
        context = super(ProjectsView, self).get_context_data(**kwargs)
        context['invites'] = Invite.objects.filter(user=self.request.user)
        context['is_site_manager'] = is_site_manager(self.request.user)
        return context


class SiteProjectsView(LoginRequiredMixin, FilterView):
    template_name = 'projects/site_projects.html'
    context_object_name = 'projects'
    paginate_by = 20
    filterset_class = ProjectFilter
    model = Project

    def get_queryset(self):
        if is_site_manager(self.request.user):
            # prepare projects queryset for the site manager
            queryset = Project.objects.filter_current_site()

            # prepare subquery for last_changed
            last_changed_subquery = models.Subquery(
                Value.objects.filter(project=models.OuterRef('pk')).order_by('-updated').values('updated')[:1]
            )
            queryset = queryset.annotate(last_changed=models.functions.Greatest('updated', last_changed_subquery))

            return queryset
        else:
            raise PermissionDenied()


class ProjectDetailView(ObjectPermissionMixin, DetailView):
    model = Project
    queryset = Project.objects.prefetch_related(
        'issues',
        'issues__task',
        'issues__task__conditions',
        'issues__task__conditions__source',
        'issues__task__conditions__target_option',
        'tasks',
        'views',
        'values'
    )
    permission_required = 'projects.view_project_object'

    def get_context_data(self, **kwargs):
        context = super(ProjectDetailView, self).get_context_data(**kwargs)
        project = context['project']
        ancestors = project.get_ancestors(include_self=True)
        values = project.values.filter(snapshot=None).select_related('attribute', 'option')
        highest = Membership.objects.filter(project__in=ancestors, user_id=OuterRef('user_id')).order_by('-project__level')
        memberships = Membership.objects.filter(project__in=ancestors) \
                                        .annotate(highest=Subquery(highest.values('project__level')[:1])) \
                                        .filter(highest=F('project__level')) \
                                        .select_related('user')

        integrations = Integration.objects.filter(project__in=ancestors)
        context['catalogs'] = Catalog.objects.filter_current_site() \
                                             .filter_group(self.request.user) \
                                             .filter_availability(self.request.user)
        context['memberships'] = memberships.order_by('user__last_name', '-project__level')
        context['integrations'] = integrations.order_by('provider_key', '-project__level')
        context['providers'] = get_plugins('SERVICE_PROVIDERS')
        context['issues'] = [issue for issue in project.issues.all() if issue.resolve(values)]
        context['snapshots'] = project.snapshots.all()
        context['invites'] = project.invites.all()
        context['membership'] = Membership.objects.filter(project=project, user=self.request.user).first()

        return context


class ProjectDeleteView(ObjectPermissionMixin, RedirectViewMixin, DeleteView):
    model = Project
    queryset = Project.objects.all()
    success_url = reverse_lazy('projects')
    permission_required = 'projects.delete_project_object'


class ProjectJoinView(LoginRequiredMixin, RedirectViewMixin, TemplateView):
    template_name = 'core/error.html'

    def get(self, request, token):
        try:
            invite = Invite.objects.get(token=token)

            if invite.is_expired:
                error = _('Sorry, your invitation has been expired.')
                invite.delete()
            elif invite.user and invite.user != request.user:
                error = _('Sorry, but this invitation is for the user "%s".' % invite.user)
            elif Membership.objects.filter(project=invite.project, user=request.user).exists():
                invite.delete()
                return redirect(invite.project.get_absolute_url())
            else:
                Membership.objects.create(
                    project=invite.project,
                    user=request.user,
                    role=invite.role
                )
                invite.delete()
                return redirect(invite.project.get_absolute_url())

        except Invite.DoesNotExist:
            error = _('Sorry, the invitation link is not valid.')

        return self.render_to_response({
            'title': _('Error'),
            'errors': [error]
        })


class ProjectCancelView(LoginRequiredMixin, RedirectViewMixin, TemplateView):
    template_name = 'core/error.html'
    success_url = reverse_lazy('projects')

    def get(self, request, token=None):
        invite = get_object_or_404(Invite, token=token)
        if invite.user in [None, request.user]:
            invite.delete()

        return redirect(self.success_url)


class ProjectLeaveView(ObjectPermissionMixin, RedirectViewMixin, FormMixin, DetailView):
    model = Project
    form_class = Form
    queryset = Project.objects.all()
    success_url = reverse_lazy('projects')
    permission_required = 'projects.leave_project_object'
    template_name = 'projects/project_confirm_leave.html'

    def post(self, request, *args, **kwargs):
        form = self.get_form()
        if form.is_valid() and 'cancel' not in request.POST:
            membership = Membership.objects.filter(project=self.get_object()).get(user=request.user)
            if not membership.is_last_owner:
                membership.delete()

        return redirect(self.success_url)


class ProjectExportView(ObjectPermissionMixin, DetailView):
    model = Project
    queryset = Project.objects.all()
    permission_required = 'projects.export_project_object'

    def render_to_response(self, context, **response_kwargs):
        export_plugin = get_plugin('PROJECT_EXPORTS', self.kwargs.get('format'))
        if export_plugin is None:
            raise Http404

        export_plugin.project = context['project']
        export_plugin.snapshot = None

        return export_plugin.render()


class ProjectQuestionsView(ObjectPermissionMixin, DetailView):
    model = Project
    queryset = Project.objects.all()
    permission_required = 'projects.view_project_object'
    template_name = 'projects/project_questions.html'

    @method_decorator(ensure_csrf_cookie)
    def get(self, request, *args, **kwargs):
        self.object = self.get_object()

        if self.object.catalog is None:
            return redirect('project_error', pk=self.object.pk)
        else:
            context = self.get_context_data(object=self.object)
            context['widgets'] = get_widgets()
            return self.render_to_response(context)


class ProjectErrorView(ObjectPermissionMixin, DetailView):
    model = Project
    queryset = Project.objects.all()
    permission_required = 'projects.view_project_object'
    template_name = 'projects/project_error.html'



from django.utils.translation import gettext_lazy as _

WIDGET_TYPES = (
    ('text', _('Text')),
    ('textarea', _('Textarea')),
    ('yesno', _('Yes/No')),
    ('checkbox', _('Checkboxes')),
    ('radio', _('Radio buttons')),
    ('select', _('Select drop-down')),
    ('autocomplete', _('Autocomplete')),
    ('range', _('Range slider')),
    ('date', _('Date picker')),
    ('file', _('File upload'))
)



# Generated by Django 2.2.16 on 2020-11-10 14:32

import django.db.models.deletion
import rdmo.projects.models.value
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0044_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='value',
            name='file',
            field=models.FileField(blank=True, help_text='The file stored for this value.', null=True, upload_to=rdmo.projects.models.value.get_file_upload_to, verbose_name='File'),
        ),
        migrations.AlterField(
            model_name='value',
            name='option',
            field=models.ForeignKey(blank=True, help_text='The option stored for this value.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='values', to='options.Option', verbose_name='Option'),
        ),
        migrations.AlterField(
            model_name='value',
            name='value_type',
            field=models.CharField(choices=[('text', 'Text'), ('url', 'URL'), ('integer', 'Integer'), ('float', 'Float'), ('boolean', 'Boolean'), ('datetime', 'Datetime'), ('option', 'Option'), ('file', 'File')], default='text', help_text='Type of this value.', max_length=8, verbose_name='Value type'),
        ),
    ]



from django.contrib.auth.models import Group
from django.core.management.base import BaseCommand

from rdmo.accounts.settings import GROUPS
from rdmo.accounts.utils import set_group_permissions


class Command(BaseCommand):

    def handle(self, *args, **options):

        for name, permissions in GROUPS:
            group, created = Group.objects.get_or_create(name=name)

            if created:
                print('Group "%s" created' % name)
            else:
                group.permissions.clear()

        set_group_permissions()



# Generated by Django 2.2.13 on 2020-08-31 16:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0035_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='project',
            name='tasks',
        ),
    ]



from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils.crypto import salted_hmac
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from ..constants import ROLE_CHOICES


class Invite(models.Model):

    key_salt = 'rdmo.projects.models.invite.Invite'

    project = models.ForeignKey(
        'Project', on_delete=models.CASCADE, related_name='invites',
        verbose_name=_('Project'),
        help_text=_('The project for this invite.')
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True,
        verbose_name=_('User'),
        help_text=_('The user for this membership.')
    )
    email = models.EmailField(
        blank=True,
        verbose_name=_('Email'),
        help_text=_('The email for this membership.')
    )
    role = models.CharField(
        max_length=12, choices=ROLE_CHOICES,
        verbose_name=_('Role'),
        help_text=_('The role for this invite.')
    )
    token = models.CharField(
        max_length=20,
        verbose_name=_('Token'),
        help_text=_('The token for this invite.')
    )
    timestamp = models.DateTimeField(
        verbose_name=_('Timestamp'),
        help_text=_('The timestamp for this invite.')
    )

    class Meta:
        ordering = ('timestamp', )
        verbose_name = _('Invite')
        verbose_name_plural = _('Invites')

    def __str__(self):
        return '%s / %s / %s' % (self.project.title, self.email, self.role)

    def save(self, *args, **kwargs):
        if self.timestamp is None:
            self.timestamp = now()
        super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('project', kwargs={'pk': self.project.pk})

    @property
    def is_expired(self):
        if settings.PROJECT_INVITE_TIMEOUT is None:
            return False
        else:
            return (now() - self.timestamp).total_seconds() > settings.PROJECT_INVITE_TIMEOUT

    def make_token(self):
        self.token = salted_hmac(self.key_salt, self._make_hash_value()).hexdigest()[::2]

    def _make_hash_value(self):
        return str(self.project_id) + str(self.email) + str(self.role) + str(self.timestamp)



import pytest
from django.urls import reverse

users = (
    ('editor', 'editor'),
    ('reviewer', 'reviewer'),
    ('user', 'user'),
    ('api', 'api'),
)


def test_home_anonymous(db, client):
    response = client.get(reverse('home'))
    assert response.status_code == 200


@pytest.mark.parametrize("username,password", users)
def test_home_user(db, client, username, password):
    client.login(username=username, password=password)
    response = client.get(reverse('home'))
    assert response.status_code == 302
    assert response.url == reverse('projects')


def test_about_anonymous(db, client):
    response = client.get(reverse('about'))
    assert response.status_code == 302


@pytest.mark.parametrize("username,password", users)
def test_about_user(db, client, username, password):
    client.login(username=username, password=password)
    response = client.get(reverse('about'))
    assert response.status_code == 200


def test_i18n_switcher(db, client):
    # get the url to switch to german
    url = reverse('i18n_switcher', args=['de'])

    # switch to german and check if the header is there
    response = client.get(url, HTTP_REFERER='http://testserver/')
    assert response.status_code == 302
    assert 'de' in response['Content-Language']

    # get the url to switch to english
    url = reverse('i18n_switcher', args=['en'])

    # switch to german and check if the header is there
    response = client.get(url)
    assert response.status_code == 302
    assert 'en' in response['Content-Language']



from rdmo.core.renderers import BaseXMLRenderer


class DomainRenderer(BaseXMLRenderer):

    def render_attribute(self, xml, attribute):
        xml.startElement('attribute', {'dc:uri': attribute['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, attribute['uri_prefix'])
        self.render_text_element(xml, 'key', {}, attribute['key'])
        self.render_text_element(xml, 'path', {}, attribute['path'])
        self.render_text_element(xml, 'dc:comment', {}, attribute['comment'])
        self.render_text_element(xml, 'parent', {'dc:uri': attribute['parent']}, None)
        xml.endElement('attribute')

        if 'children' in attribute and attribute['children']:
            for child in attribute['children']:
                self.render_attribute(xml, child)


class AttributeRenderer(DomainRenderer):

    def render_document(self, xml, attributes):
        xml.startElement('rdmo', {
            'xmlns:dc': "http://purl.org/dc/elements/1.1/",
            'created': self.created
        })
        for attribute in attributes:
            self.render_attribute(xml, attribute)
        xml.endElement('rdmo')



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-09 12:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0002_meta'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='description',
            field=models.TextField(blank=True, help_text='Optional', verbose_name='description'),
        ),
    ]



import io

import pytest
from django.core.management import call_command
from django.core.management.base import CommandError

prune_assignments = [
    ('guest', "['owner', 'guest', 'author', 'manager']", [6]),
    ('author', "['owner', 'author', 'manager']", [6, 9]),
    ('manager', "['owner', 'manager']", [6, 8, 9]),
    ('owner', "['owner']", [6, 7, 8, 9])
]

def get_prune_output(projects, remove=False):
    project_output = ""
    for proj in projects:
        project_output += "Prune Test (id=%i)\n" % proj
        if remove:
            project_output += "...removing...OK\n"
    return project_output


def test_prune_projects_error(db, settings):
    stdout, stderr = io.StringIO(), io.StringIO()

    with pytest.raises(CommandError) as e:
        call_command('prune_projects', '--min_role', 'unknown', stdout=stdout, stderr=stderr)

    assert str(e.value) == 'Role "unknown" does not exist'


@pytest.mark.parametrize('min_role,role_list,projects', prune_assignments)
def test_prune_projects_output(db, settings, min_role, role_list, projects):
    stdout, stderr = io.StringIO(), io.StringIO()

    call_command('prune_projects', '--min_role', min_role, stdout=stdout, stderr=stderr)

    assert stdout.getvalue() == \
        "Found projects without %s:\n%s" % (role_list, get_prune_output(projects))
    assert not stderr.getvalue()


def test_prune_projects_output2(db, settings):
    stdout, stderr = io.StringIO(), io.StringIO()

    call_command('prune_projects', stdout=stdout, stderr=stderr)

    assert stdout.getvalue() == \
        "Found projects without ['owner']:\n%s" % (get_prune_output([6, 7, 8, 9]))
    assert not stderr.getvalue()


def test_prune_projects_remove(db, settings):
    stdout, stderr = io.StringIO(), io.StringIO()

    call_command('prune_projects', '--remove', stdout=stdout, stderr=stderr)

    assert stdout.getvalue() == \
        "Found projects without ['owner']:\n%s" % (get_prune_output([6, 7, 8, 9], True))
    assert not stderr.getvalue()

    stdout, stderr = io.StringIO(), io.StringIO()
    call_command('prune_projects', '--remove', stdout=stdout, stderr=stderr)

    assert stdout.getvalue() == "No projects without ['owner']\n"
    assert not stderr.getvalue()



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _
from mptt.models import MPTTModel, TreeForeignKey

from rdmo.core.utils import copy_model, join_url


class Attribute(MPTTModel):

    uri = models.URLField(
        max_length=640, blank=True,
        verbose_name=_('URI'),
        help_text=_('The Uniform Resource Identifier of this attribute (auto-generated).')
    )
    uri_prefix = models.URLField(
        max_length=256,
        verbose_name=_('URI Prefix'),
        help_text=_('The prefix for the URI of this attribute.')
    )
    key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Key'),
        help_text=_('The internal identifier of this attribute.')
    )
    path = models.CharField(
        max_length=512, db_index=True,
        verbose_name=_('Path'),
        help_text=_('The path part of the URI of this attribute (auto-generated).')
    )
    comment = models.TextField(
        blank=True,
        verbose_name=_('Comment'),
        help_text=_('Additional information about this attribute.')
    )
    locked = models.BooleanField(
        default=False,
        verbose_name=_('Locked'),
        help_text=_('Designates whether this attribute (and it\'s descendants) can be changed.')
    )
    parent = TreeForeignKey(
        'self', null=True, blank=True,
        on_delete=models.CASCADE, related_name='children', db_index=True,
        verbose_name=_('Parent attribute'),
        help_text=_('Parent attribute in the domain model.')
    )

    class Meta:
        ordering = ('uri', )
        verbose_name = _('Attribute')
        verbose_name_plural = _('Attributes')

    def __str__(self):
        return self.path

    def save(self, *args, **kwargs):
        self.path = self.build_path(self.key, self.parent)
        self.uri = self.build_uri(self.uri_prefix, self.path)
        super().save(*args, **kwargs)

        # recursively save children
        for child in self.children.all():
            child.save()

    def copy(self, uri_prefix, key, parent=None, rebuild=True):
        assert parent not in self.get_descendants(include_self=True)

        # copy the attribute
        attribute = copy_model(self, uri_prefix=uri_prefix, key=key, parent=parent or self.parent)

        # recursively copy children
        for child in self.children.all():
            child.copy(uri_prefix, child.key, parent=attribute, rebuild=False)

        # rebuild the mptt tree, but only for the initially copied attribute
        if rebuild:
            Attribute.objects.rebuild()

        return attribute

    @property
    def parent_fields(self):
        return ('parent', )

    @property
    def is_locked(self):
        return self.get_ancestors(include_self=True).filter(locked=True).exists()

    @classmethod
    def build_path(cls, key, parent):
        assert key
        path = key
        while parent:
            path = parent.key + '/' + path
            parent = parent.parent
        return path

    @classmethod
    def build_uri(cls, uri_prefix, path):
        assert path
        return join_url(uri_prefix or settings.DEFAULT_URI_PREFIX, '/domain/', path)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-05 14:54
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0039_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='catalog',
            name='title_lang3',
            field=models.CharField(blank=True, help_text='The title for this catalog in the tertiary language.', max_length=256, null=True, verbose_name='Title (tertiary)'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='title_lang4',
            field=models.CharField(blank=True, help_text='The title for this catalog in the quaternary language.', max_length=256, null=True, verbose_name='Title (quaternary)'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='title_lang5',
            field=models.CharField(blank=True, help_text='The title for this catalog in the quinary language.', max_length=256, null=True, verbose_name='Title (quinary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='help_lang3',
            field=models.TextField(blank=True, help_text='The help text for this question in the tertiary language.', null=True, verbose_name='Help (tertiary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='help_lang4',
            field=models.TextField(blank=True, help_text='The help text for this question in the quaternary language.', null=True, verbose_name='Help (quaternary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='help_lang5',
            field=models.TextField(blank=True, help_text='The help text for this question in the quinary language.', null=True, verbose_name='Help (quinary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='text_lang3',
            field=models.TextField(blank=True, help_text='The text for this question in the tertiary language.', null=True, verbose_name='Text (tertiary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='text_lang4',
            field=models.TextField(blank=True, help_text='The text for this question in the quaternary language.', null=True, verbose_name='Text (quaternary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='text_lang5',
            field=models.TextField(blank=True, help_text='The text for this question in the quinary language.', null=True, verbose_name='Text (quinary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_lang3',
            field=models.CharField(blank=True, help_text='The name displayed for this question in the tertiary language.', max_length=256, verbose_name='Name (tertiary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_lang4',
            field=models.CharField(blank=True, help_text='The name displayed for this question in the quaternary language.', max_length=256, verbose_name='Name (quaternary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_lang5',
            field=models.CharField(blank=True, help_text='The name displayed for this question in the quinary language.', max_length=256, verbose_name='Name (quinary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_plural_lang3',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the tertiary language.', max_length=256, verbose_name='Plural name (tertiary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_plural_lang4',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the quaternary language.', max_length=256, verbose_name='Plural name (quaternary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_plural_lang5',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the quinary language.', max_length=256, verbose_name='Plural name (quinary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='help_lang3',
            field=models.TextField(blank=True, help_text='The help text for this questionset in the tertiary language.', null=True, verbose_name='Help (tertiary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='help_lang4',
            field=models.TextField(blank=True, help_text='The help text for this questionset in the quaternary language.', null=True, verbose_name='Help (quaternary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='help_lang5',
            field=models.TextField(blank=True, help_text='The help text for this questionset in the quinary language.', null=True, verbose_name='Help (quinary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='title_lang3',
            field=models.CharField(blank=True, help_text='The title for this questionset in the tertiary language.', max_length=256, null=True, verbose_name='Title (tertiary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='title_lang4',
            field=models.CharField(blank=True, help_text='The title for this questionset in the quaternary language.', max_length=256, null=True, verbose_name='Title (quaternary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='title_lang5',
            field=models.CharField(blank=True, help_text='The title for this questionset in the quinary language.', max_length=256, null=True, verbose_name='Title (quinary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_lang3',
            field=models.CharField(blank=True, help_text='The  name displayed for this question in the tertiary language.', max_length=256, verbose_name='Name (tertiary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_lang4',
            field=models.CharField(blank=True, help_text='The  name displayed for this question in the quaternary language.', max_length=256, verbose_name='Name (quaternary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_lang5',
            field=models.CharField(blank=True, help_text='The  name displayed for this question in the quinary language.', max_length=256, verbose_name='Name (quinary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_plural_lang3',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the tertiary language.', max_length=256, verbose_name='Plural name (tertiary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_plural_lang4',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the quaternary language.', max_length=256, verbose_name='Plural name (quaternary)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_plural_lang5',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question in the quinary language.', max_length=256, verbose_name='Plural name (quinary)'),
        ),
        migrations.AddField(
            model_name='section',
            name='title_lang3',
            field=models.CharField(blank=True, help_text='The title for this section in the tertiary language.', max_length=256, null=True, verbose_name='Title (tertiary)'),
        ),
        migrations.AddField(
            model_name='section',
            name='title_lang4',
            field=models.CharField(blank=True, help_text='The title for this section in the quaternary language.', max_length=256, null=True, verbose_name='Title (quaternary)'),
        ),
        migrations.AddField(
            model_name='section',
            name='title_lang5',
            field=models.CharField(blank=True, help_text='The title for this section in the quinary language.', max_length=256, null=True, verbose_name='Title (quinary)'),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='title_lang1',
            field=models.CharField(blank=True, help_text='The title for this catalog in the primary language.', max_length=256, null=True, verbose_name='Title (primary)'),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='title_lang2',
            field=models.CharField(blank=True, help_text='The title for this catalog in the secondary language.', max_length=256, null=True, verbose_name='Title (secondary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='text_lang1',
            field=models.TextField(blank=True, help_text='The text for this question in the primary language.', null=True, verbose_name='Text (primary)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='text_lang2',
            field=models.TextField(blank=True, help_text='The text for this question in the secondary language.', null=True, verbose_name='Text (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang1',
            field=models.CharField(blank=True, help_text='The title for this questionset in the primary language.', max_length=256, null=True, verbose_name='Title (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang2',
            field=models.CharField(blank=True, help_text='The title for this questionset in the secondary language.', max_length=256, null=True, verbose_name='Title (secondary)'),
        ),
        migrations.AlterField(
            model_name='section',
            name='title_lang1',
            field=models.CharField(blank=True, help_text='The title for this section in the primary language.', max_length=256, null=True, verbose_name='Title (primary)'),
        ),
        migrations.AlterField(
            model_name='section',
            name='title_lang2',
            field=models.CharField(blank=True, help_text='The title for this section in the secondary language.', max_length=256, null=True, verbose_name='Title (secondary)'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-27 15:24
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0006_refactoring'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='option',
            options={'ordering': ('uri',), 'verbose_name': 'Option', 'verbose_name_plural': 'Options'},
        ),
        migrations.AlterModelOptions(
            name='optionset',
            options={'ordering': ('uri',), 'verbose_name': 'Option set', 'verbose_name_plural': 'Option sets'},
        ),
    ]



from collections import defaultdict

from django.contrib.sites.shortcuts import get_current_site
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.utils.translation import gettext_lazy as _

from rdmo.core.imports import handle_uploaded_file
from rdmo.core.plugins import get_plugin, get_plugins
from rdmo.questions.models import Question

from .models import Membership, Project
from .utils import (save_import_snapshot_values, save_import_tasks,
                    save_import_values, save_import_views)


class ProjectImportMixin(object):

    def get_current_values(self, current_project):
        queryset = current_project.values.filter(snapshot=None).select_related('attribute', 'option')

        current_values = defaultdict(lambda: defaultdict(lambda: defaultdict(lambda: defaultdict(list))))
        for value in queryset:
            current_values[value.attribute.uri][value.set_prefix][value.set_index][value.collection_index] = value
        return current_values

    def get_questions(self, catalog):
        queryset = Question.objects.filter(questionset__section__catalog=catalog) \
                                   .select_related('attribute') \
                                   .order_by('attribute__uri')

        questions = {}
        for question in queryset:
            if question.attribute.uri not in questions:
                questions[question.attribute.uri] = question
        return questions

    def update_values(self, current_project, catalog, values, snapshots=[]):
        questions = self.get_questions(catalog)
        current_values = self.get_current_values(current_project) if current_project else {}

        for value in values:
            if value.attribute:
                value.pk = None
                value.question = questions.get(value.attribute.uri)
                value.current = current_values.get(value.attribute.uri, {}) \
                                              .get(value.set_prefix, {}) \
                                              .get(value.set_index, {}) \
                                              .get(value.collection_index)

        for snapshot in snapshots:
            snapshot.pk = None

            for value in snapshot.snapshot_values:
                if value.attribute:
                    value.pk = None
                    value.question = questions.get(value.attribute.uri)
                    value.current = current_values.get(value.attribute.uri, {}) \
                                                  .get(value.set_prefix, {}) \
                                                  .get(value.set_index, {}) \
                                                  .get(value.collection_index)

    def upload_file(self):
        current_project = self.object

        try:
            uploaded_file = self.request.FILES['uploaded_file']
        except KeyError:
            return None
        else:
            import_tmpfile_name = handle_uploaded_file(uploaded_file)

        for import_key, import_plugin in get_plugins('PROJECT_IMPORTS').items():
            import_plugin.file_name = import_tmpfile_name
            import_plugin.current_project = current_project

            if import_plugin.check():
                try:
                    import_plugin.process()
                except ValidationError as e:
                    return render(self.request, 'core/error.html', {
                        'title': _('Import error'),
                        'errors': e
                    }, status=400)

                # store information in session for ProjectCreateImportView
                self.request.session['update_import_tmpfile_name'] = import_tmpfile_name
                self.request.session['update_import_key'] = import_key

                # attach questions and current values
                self.update_values(current_project, import_plugin.catalog, import_plugin.values, import_plugin.snapshots)

                return render(self.request, 'projects/project_import.html', {
                    'method': 'import_file',
                    'current_project': current_project,
                    'source_title': uploaded_file.name,
                    'source_project': import_plugin.project,
                    'values': import_plugin.values,
                    'snapshots': import_plugin.snapshots if not current_project else None,
                    'tasks': import_plugin.tasks,
                    'views': import_plugin.views
                })

        return render(self.request, 'core/error.html', {
            'title': _('Import error'),
            'errors': [_('Files of this type cannot be imported.')]
        }, status=400)

    def import_file(self):
        current_project = self.object

        import_tmpfile_name = self.request.session.get('update_import_tmpfile_name')
        import_key = self.request.session.get('update_import_key')
        checked = [key for key, value in self.request.POST.items() if 'on' in value]

        if import_tmpfile_name and import_key:
            import_plugin = get_plugin('PROJECT_IMPORTS', import_key)
            import_plugin.file_name = import_tmpfile_name
            import_plugin.current_project = current_project

            if import_plugin.check():
                try:
                    import_plugin.process()
                except ValidationError as e:
                    return render(self.request, 'core/error.html', {
                        'title': _('Import error'),
                        'errors': e
                    }, status=400)

                # attach questions and current values
                self.update_values(current_project, import_plugin.catalog, import_plugin.values, import_plugin.snapshots)

                if current_project:
                    save_import_values(self.object, import_plugin.values, checked)
                    save_import_tasks(self.object, import_plugin.tasks)
                    save_import_views(self.object, import_plugin.views)

                    return HttpResponseRedirect(current_project.get_absolute_url())

                else:
                    # add current site and save project
                    import_plugin.project.site = get_current_site(self.request)
                    import_plugin.project.save()

                    # add user to project
                    membership = Membership(project=import_plugin.project, user=self.request.user, role='owner')
                    membership.save()

                    for value in import_plugin.values:
                        value.current = None

                    save_import_values(import_plugin.project, import_plugin.values, checked)
                    save_import_snapshot_values(import_plugin.project, import_plugin.snapshots, checked)
                    save_import_tasks(import_plugin.project, import_plugin.tasks)
                    save_import_views(import_plugin.project, import_plugin.views)

                    return HttpResponseRedirect(import_plugin.project.get_absolute_url())

    def import_project(self):
        current_project = self.object

        # get the original project
        project = get_object_or_404(Project.objects.all(), id=self.request.POST.get('source'))

        # check if the user has the permission to access the original project
        if not self.request.user.has_perm('projects.view_project_object', project):
            self.handle_no_permission()

        values = project.values.filter(snapshot=None) \
                               .select_related('attribute', 'option')

        # attach questions and current values
        self.update_values(current_project, current_project.catalog, values)

        checked = [key for key, value in self.request.POST.items() if 'on' in value]
        if checked:
            save_import_values(current_project, values, checked)
            return HttpResponseRedirect(current_project.get_absolute_url())

        else:
            return render(self.request, 'projects/project_import.html', {
                'method': 'import_project',
                'source': project.id,
                'source_title': project.title,
                'current_project': current_project,
                'values': values,
            })



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-31 10:44
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0012_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='option',
            options={'ordering': ('optionset__order', 'optionset__key', 'order', 'key'), 'permissions': (('view_option', 'Can view Option'),), 'verbose_name': 'Option', 'verbose_name_plural': 'Options'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-07-26 15:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0002_many_to_many_for_conditions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='conditions',
            field=models.ManyToManyField(blank=True, to='conditions.Condition'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-21 09:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0018_validator'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='attribute',
            managers=[
            ],
        ),
        migrations.AlterModelManagers(
            name='attributeentity',
            managers=[
            ],
        ),
    ]



import base64
import hashlib
import os
import shutil

import requests
from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def handle(self, *args, **options):
        # remove old vendor files
        vendor_dir = os.path.join(settings.BASE_DIR, 'vendor/')
        try:
            shutil.rmtree(vendor_dir)
        except FileNotFoundError:
            pass

        for key, vendor_conf in settings.VENDOR.items():
            for file_type in ['js', 'css', 'img', 'font']:
                if file_type in vendor_conf:
                    for file in vendor_conf[file_type]:
                        # get the directory and the file_name
                        path_tokens = ['vendor', key] + os.path.normpath(file['path']).split(os.path.sep)
                        directory = os.path.join(*path_tokens[:-1])
                        file_name = os.path.join(*path_tokens)

                        # create the needed diredtories
                        try:
                            os.makedirs(directory)
                        except OSError:
                            pass

                        # get the full url of the file
                        url = requests.compat.urljoin(vendor_conf['url'], file['path'])

                        print('%s -> %s' % (url, file_name))

                        # fetch the file from the cdn
                        response = requests.get(url)
                        response.raise_for_status()
                        with open(file_name, 'wb') as f:
                            f.write(response.content)

                        # check the intergrety of the file if a SRI was supplied
                        if 'sri' in file:
                            algorithm, file_hash = file['sri'].split('-')

                            h = hashlib.new(algorithm)
                            h.update(open(file_name, 'rb').read())
                            if base64.b64encode(h.digest()).decode() != file_hash:
                                raise Exception('Subresource Integrity (SRI) failed for %s' % file_name)



from django.contrib.auth.models import Group
from django.contrib.sites.models import Site
from rest_framework import serializers

from rdmo.core.utils import get_languages, markdown2html


class RecursiveField(serializers.Serializer):

    def to_representation(self, value):
        serializer = self.parent.parent.__class__(value, context=self.context)
        return serializer.data


class ChoicesSerializer(serializers.Serializer):
    id = serializers.SerializerMethodField()
    text = serializers.SerializerMethodField()

    def get_id(self, obj):
        return obj[0]

    def get_text(self, obj):
        return obj[1]


class MarkdownSerializerMixin(serializers.Serializer):

    markdown_fields = ()

    def to_representation(self, instance):
        response = super(MarkdownSerializerMixin, self).to_representation(instance)

        for markdown_field in self.markdown_fields:
            if markdown_field in response and response[markdown_field]:
                response[markdown_field] = markdown2html(response[markdown_field])

        return response


class TranslationSerializerMixin(object):

    def __init__(self, *args, **kwargs):
        super(TranslationSerializerMixin, self).__init__(*args, **kwargs)

        meta = getattr(self, 'Meta', None)
        if meta:
            for lang_code, lang_string, lang_field in get_languages():
                for field in meta.trans_fields:
                    field_name = '%s_%s' % (field, lang_field)
                    model_field = meta.model._meta.get_field(field_name)

                    self.fields['%s_%s' % (field, lang_code)] = serializers.CharField(
                        source=field_name,
                        required=not model_field.blank,
                        allow_null=model_field.null,
                        allow_blank=model_field.blank)


class SiteSerializer(serializers.ModelSerializer):

    class Meta:
        model = Site
        fields = (
            'id',
            'domain',
            'name'
        )


class GroupSerializer(serializers.ModelSerializer):

    class Meta:
        model = Group
        fields = (
            'id',
            'name'
        )



from rdmo.core.plugins import Plugin


class Provider(Plugin):

    search = False

    def get_options(self, project, search=None):
        raise NotImplementedError


class SimpleProvider(Provider):

    def get_options(self, project, search=None):
        return [
            {
                'id': 'simple_1',
                'text': 'Simple answer 1'
            },
            {
                'id': 'simple_2',
                'text': 'Simple answer 2'
            },
            {
                'id': 'simple_3',
                'text': 'Simple answer 3'
            }
        ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-01-30 14:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0015_rename_de_to_lang2'),
    ]

    operations = [
        migrations.AlterField(
            model_name='option',
            name='text_lang1',
            field=models.CharField(help_text='The text for this option in the primary language.', max_length=256, verbose_name='Text (primary)'),
        ),
        migrations.AlterField(
            model_name='option',
            name='text_lang2',
            field=models.CharField(help_text='The text for this option in the secondary language.', max_length=256, verbose_name='Text (secondary)'),
        ),
    ]



import pytest
from django.urls import reverse

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('api', 'api'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'owner': 200, 'manager': 200, 'author': 200, 'guest': 200, 'api': 200, 'user': 200, 'anonymous': 401
    }
}

urlnames = {
    'list': 'setting-list',
}


@pytest.mark.parametrize('username,password', users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()



from django.conf import settings
from django.core import mail

from rdmo.core.mail import send_mail


def test_send_mail(db):
    send_mail('Subject', 'Message', to=['user@example.com'])

    assert len(mail.outbox) == 1
    assert mail.outbox[0].subject == '[example.com] Subject'
    assert mail.outbox[0].body == 'Message'
    assert mail.outbox[0].from_email == settings.DEFAULT_FROM_EMAIL
    assert mail.outbox[0].to == ['user@example.com']
    assert mail.outbox[0].cc == []
    assert mail.outbox[0].bcc == []
    assert mail.outbox[0].attachments == []


def test_send_mail_cc(db):
    send_mail('Subject', 'Message', to=['user@example.com'], cc=['user2@example.com'])

    assert len(mail.outbox) == 1
    assert mail.outbox[0].subject == '[example.com] Subject'
    assert mail.outbox[0].body == 'Message'
    assert mail.outbox[0].from_email == settings.DEFAULT_FROM_EMAIL
    assert mail.outbox[0].to == ['user@example.com']
    assert mail.outbox[0].cc == ['user2@example.com']
    assert mail.outbox[0].bcc == []
    assert mail.outbox[0].attachments == []


def test_send_mail_bcc(db):
    send_mail('Subject', 'Message', bcc=['user2@example.com'])

    assert len(mail.outbox) == 1
    assert mail.outbox[0].subject == '[example.com] Subject'
    assert mail.outbox[0].body == 'Message'
    assert mail.outbox[0].from_email == settings.DEFAULT_FROM_EMAIL
    assert mail.outbox[0].to == []
    assert mail.outbox[0].cc == []
    assert mail.outbox[0].bcc == ['user2@example.com']
    assert mail.outbox[0].attachments == []


def test_send_mail_from_email(db):
    send_mail('Subject', 'Message', from_email='user@example.com', to=['user2@example.com'])

    assert len(mail.outbox) == 1
    assert mail.outbox[0].subject == '[example.com] Subject'
    assert mail.outbox[0].body == 'Message'
    assert mail.outbox[0].from_email == 'user@example.com'
    assert mail.outbox[0].to == ['user2@example.com']
    assert mail.outbox[0].cc == []
    assert mail.outbox[0].bcc == []
    assert mail.outbox[0].attachments == []


def test_send_mail_from_attachments(db):
    send_mail('Subject', 'Message', to=['user2@example.com'], attachments=[
        ('Attachment', b'attachment', 'plain/text')
    ])

    assert len(mail.outbox) == 1
    assert mail.outbox[0].subject == '[example.com] Subject'
    assert mail.outbox[0].body == 'Message'
    assert mail.outbox[0].from_email == settings.DEFAULT_FROM_EMAIL
    assert mail.outbox[0].to == ['user2@example.com']
    assert mail.outbox[0].cc == []
    assert mail.outbox[0].bcc == []
    assert mail.outbox[0].attachments == [
        ('Attachment', b'attachment', 'plain/text')
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-28 12:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0006_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='task',
            options={'ordering': ('uri',), 'permissions': (('view_task', 'Can view Task'),), 'verbose_name': 'Task', 'verbose_name_plural': 'Tasks'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-21 14:11
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('projects', '0011_refactoring'),
    ]

    operations = [
        migrations.CreateModel(
            name='Membership',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', models.CharField(choices=[('owner', 'Owner'), ('manager', 'Manager'), ('author', 'Author'), ('guest', 'Guest')], help_text='The role for this membership.', max_length=12, verbose_name='Role')),
            ],
            options={
                'ordering': ('project__title',),
                'verbose_name': 'Membership',
                'verbose_name_plural': 'Memberships',
            },
        ),
        migrations.AlterField(
            model_name='project',
            name='owner',
            field=models.ManyToManyField(help_text='The list of owners for this project.', related_name='_project_owner_+', to=settings.AUTH_USER_MODEL, verbose_name='Owner'),
        ),
        migrations.AddField(
            model_name='membership',
            name='project',
            field=models.ForeignKey(help_text='The project for this membership.', on_delete=django.db.models.deletion.CASCADE, to='projects.Project', verbose_name='Project'),
        ),
        migrations.AddField(
            model_name='membership',
            name='user',
            field=models.ForeignKey(help_text='The user for this membership.', on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
        migrations.AddField(
            model_name='project',
            name='user',
            field=models.ManyToManyField(help_text='The list of users for this project.', through='projects.Membership', to=settings.AUTH_USER_MODEL, verbose_name='User'),
        ),
    ]



from rest_framework import serializers
from rest_framework.reverse import reverse

from rdmo.domain.models import Attribute
from rdmo.options.models import OptionSet
from rdmo.questions.models import Question, QuestionSet
from rdmo.tasks.models import Task

from ..models import Condition
from ..validators import ConditionLockedValidator, ConditionUniqueURIValidator


class OptionSetSerializer(serializers.ModelSerializer):

    class Meta:
        model = OptionSet
        fields = (
            'id',
            'uri'
        )


class QuestionSetSerializer(serializers.ModelSerializer):

    class Meta:
        model = QuestionSet
        fields = (
            'id',
            'uri'
        )


class QuestionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Question
        fields = (
            'id',
            'uri'
        )


class TaskSerializer(serializers.ModelSerializer):

    class Meta:
        model = Task
        fields = (
            'id',
            'uri'
        )


class ConditionSerializer(serializers.ModelSerializer):

    key = serializers.SlugField(required=True)
    source = serializers.PrimaryKeyRelatedField(queryset=Attribute.objects.all(), required=True)
    optionsets = OptionSetSerializer(many=True, read_only=True)
    questionsets = QuestionSetSerializer(many=True, read_only=True)
    questions = QuestionSerializer(many=True, read_only=True)
    tasks = TaskSerializer(many=True, read_only=True)

    class Meta:
        model = Condition
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'source',
            'relation',
            'target_text',
            'target_option',
            'optionsets',
            'questionsets',
            'questions',
            'tasks'
        )
        validators = (
            ConditionUniqueURIValidator(),
            ConditionLockedValidator()
        )


class ConditionIndexSerializer(serializers.ModelSerializer):

    target_option_uri = serializers.CharField(source='target_option.uri', default=None, read_only=True)
    target_option_text = serializers.CharField(source='target_option.text', default=None, read_only=True)
    xml_url = serializers.SerializerMethodField()

    class Meta:
        model = Condition
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'source_label',
            'relation_label',
            'target_text',
            'target_label',
            'target_option_uri',
            'target_option_text',
            'xml_url'
        )

    def get_xml_url(self, obj):
        return reverse('v1-conditions:condition-detail-export', args=[obj.pk])



# Generated by Django 2.2.13 on 2020-08-31 15:07

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0031_related_name'),
        ('projects', '0033_default_value_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='Issue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('open', 'open'), ('in_progress', 'in progress'), ('closed', 'closed')], default='open', help_text='The status for this issue.', max_length=12, verbose_name='Status')),
                ('project', models.ForeignKey(help_text='The project for this issue.', on_delete=django.db.models.deletion.CASCADE, related_name='issues', to='projects.Project', verbose_name='Project')),
                ('task', models.ForeignKey(help_text='The task for this issue.', on_delete=django.db.models.deletion.CASCADE, related_name='issues', to='tasks.Task', verbose_name='Task')),
            ],
            options={
                'verbose_name': 'Issue',
                'verbose_name_plural': 'Issues',
                'ordering': ('project__title',),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-28 11:23
from __future__ import unicode_literals

from django.db import migrations


def set_question_condition(apps, schema_editor):
    QuestionEntity = apps.get_model('questions', 'QuestionEntity')

    for entity in QuestionEntity.objects.all():
        try:
            for condition in entity.attribute_entity.conditions.all():
                entity.conditions.add(condition)

        except AttributeError:
            pass


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0019_questionentity_conditions'),
    ]

    operations = [
        migrations.RunPython(set_question_condition),
    ]



from django.db import models
from django.db.models import Prefetch
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from rdmo.core.constants import VALUE_TYPE_CHOICES
from rdmo.core.exports import XMLResponse
from rdmo.core.permissions import HasModelPermission
from rdmo.core.views import ChoicesViewSet
from rdmo.core.viewsets import CopyModelMixin

from .models import Catalog, Question, QuestionSet, Section
from .renderers import (CatalogRenderer, QuestionRenderer, QuestionSetRenderer,
                        SectionRenderer)
from .serializers.export import (CatalogExportSerializer,
                                 QuestionExportSerializer,
                                 QuestionSetExportSerializer,
                                 SectionExportSerializer)
from .serializers.v1 import (CatalogIndexSerializer, CatalogNestedSerializer,
                             CatalogSerializer, QuestionIndexSerializer,
                             QuestionNestedSerializer, QuestionSerializer,
                             QuestionSetIndexSerializer,
                             QuestionSetNestedSerializer,
                             QuestionSetSerializer, SectionIndexSerializer,
                             SectionNestedSerializer, SectionSerializer)
from .utils import get_widget_type_choices


class CatalogViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    serializer_class = CatalogSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'uri',
        'key',
        'comment'
    )

    def get_queryset(self):
        queryset = Catalog.objects.annotate(projects_count=models.Count('projects')) \
                                  .prefetch_related('sites', 'groups')
        if self.action in ('nested', 'detail_export'):
            return queryset.prefetch_related(
                'sections',
                Prefetch('sections__questionsets', queryset=QuestionSet.objects.filter(questionset=None).prefetch_related(
                    'conditions',
                    'questions',
                    'questions__attribute',
                    'questions__optionsets',
                    'questions__conditions',
                    'questionsets',
                    'questionsets__attribute',
                    'questionsets__conditions',
                    'questionsets__questions',
                    'questionsets__questions__attribute',
                    'questionsets__questions__optionsets',
                    'questionsets__questions__conditions',
                    'questionsets__questionsets'
                ).select_related('attribute'))
            )
        else:
            return queryset

    @action(detail=True)
    def nested(self, request, pk):
        serializer = CatalogNestedSerializer(self.get_object())
        return Response(serializer.data)

    @action(detail=False)
    def index(self, request):
        serializer = CatalogIndexSerializer(self.get_queryset(), many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = CatalogExportSerializer(self.get_queryset(), many=True)
        xml = CatalogRenderer().render(serializer.data)
        return XMLResponse(xml, name='catalogs')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        serializer = CatalogExportSerializer(self.get_object())
        xml = CatalogRenderer().render([serializer.data])
        return XMLResponse(xml, name=self.get_object().key)


class SectionViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    serializer_class = SectionSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'uri',
        'path',
        'key',
        'catalog',
        'comment'
    )

    def get_queryset(self):
        queryset = Section.objects.all()
        if self.action in ('nested', 'detail_export'):
            return queryset.prefetch_related(
                Prefetch('questionsets', queryset=QuestionSet.objects.filter(questionset=None).prefetch_related(
                    'conditions',
                    'questions',
                    'questions__attribute',
                    'questions__optionsets',
                    'questions__conditions',
                    'questionsets',
                    'questionsets__attribute',
                    'questionsets__conditions',
                    'questionsets__questions',
                    'questionsets__questions__attribute',
                    'questionsets__questions__optionsets',
                    'questionsets__questions__conditions',
                    'questionsets__questionsets'
                ).select_related('attribute'))
            )
        else:
            return queryset

    @action(detail=True)
    def nested(self, request, pk):
        serializer = SectionNestedSerializer(self.get_object())
        return Response(serializer.data)

    @action(detail=False)
    def index(self, request):
        serializer = SectionIndexSerializer(self.get_queryset(), many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = SectionExportSerializer(self.get_queryset(), many=True)
        xml = SectionRenderer().render(serializer.data)
        return XMLResponse(xml, name='sections')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        serializer = SectionExportSerializer(self.get_object())
        xml = SectionRenderer().render([serializer.data])
        return XMLResponse(xml, name=self.get_object().path)


class QuestionSetViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    serializer_class = QuestionSetSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'attribute',
        'uri',
        'path',
        'key',
        'section',
        'comment',
        'is_collection'
    )

    def get_queryset(self):
        queryset = QuestionSet.objects.all()
        if self.action in ('nested', 'detail_export'):
            return queryset.prefetch_related(
                'conditions',
                'questions',
                'questions__attribute',
                'questions__optionsets',
                'questions__conditions',
                'questionsets',
                'questionsets__attribute',
                'questionsets__conditions',
                'questionsets__questions',
                'questionsets__questions__attribute',
                'questionsets__questions__optionsets',
                'questionsets__questions__conditions',
                'questionsets__questionsets'
            ).select_related('attribute')
        else:
            return queryset

    @action(detail=True)
    def nested(self, request, pk):
        serializer = QuestionSetNestedSerializer(self.get_object())
        return Response(serializer.data)

    @action(detail=False)
    def index(self, request):
        serializer = QuestionSetIndexSerializer(self.get_queryset(), many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = QuestionSetExportSerializer(self.get_queryset(), many=True)
        xml = QuestionSetRenderer().render(serializer.data)
        return XMLResponse(xml, name='questionsets')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        serializer = QuestionSetExportSerializer(self.get_object())
        xml = QuestionSetRenderer().render([serializer.data])
        return XMLResponse(xml, name=self.get_object().path)


class QuestionViewSet(CopyModelMixin, ModelViewSet):
    permission_classes = (HasModelPermission, )
    queryset = Question.objects.all()
    serializer_class = QuestionSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'attribute',
        'uri',
        'path',
        'key',
        'questionset',
        'is_collection',
        'value_type',
        'widget_type',
        'unit',
        'comment'
    )

    def get_queryset(self):
        queryset = Question.objects.all()
        if self.action in ('nested', 'detail_export'):
            return queryset.prefetch_related(
                'optionsets',
                'conditions'
            ).select_related(
                'attribute'
            )
        else:
            return queryset

    @action(detail=True)
    def nested(self, request, pk):
        serializer = QuestionNestedSerializer(self.get_object())
        return Response(serializer.data)

    @action(detail=False)
    def index(self, request):
        serializer = QuestionIndexSerializer(self.get_queryset(), many=True)
        return Response(serializer.data)

    @action(detail=False, permission_classes=[HasModelPermission])
    def export(self, request):
        serializer = QuestionExportSerializer(self.get_queryset(), many=True)
        xml = QuestionRenderer().render(serializer.data)
        return XMLResponse(xml, name='questions')

    @action(detail=True, url_path='export', permission_classes=[HasModelPermission])
    def detail_export(self, request, pk=None):
        serializer = QuestionExportSerializer(self.get_object())
        xml = QuestionRenderer().render([serializer.data])
        return XMLResponse(xml, name=self.get_object().path)


class WidgetTypeViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    queryset = get_widget_type_choices()


class ValueTypeViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    queryset = VALUE_TYPE_CHOICES



# Generated by Django 2.2.16 on 2020-11-10 14:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0053_related_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='question',
            name='value_type',
            field=models.CharField(choices=[('text', 'Text'), ('url', 'URL'), ('integer', 'Integer'), ('float', 'Float'), ('boolean', 'Boolean'), ('datetime', 'Datetime'), ('option', 'Option'), ('file', 'File')], help_text='Type of value for this question.', max_length=8, verbose_name='Value type'),
        ),
        migrations.AlterField(
            model_name='question',
            name='widget_type',
            field=models.CharField(choices=[('text', 'Text'), ('textarea', 'Textarea'), ('yesno', 'Yes/No'), ('checkbox', 'Checkboxes'), ('radio', 'Radio buttons'), ('select', 'Select drop-down'), ('range', 'Range slider'), ('date', 'Date picker'), ('file', 'File upload')], help_text='Type of widget for this question.', max_length=12, verbose_name='Widget type'),
        ),
    ]



from rdmo.core.renderers import BaseXMLRenderer
from rdmo.core.utils import get_languages


class TasksRenderer(BaseXMLRenderer):

    def render_task(self, xml, task):
        xml.startElement('task', {'dc:uri': task['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, task['uri_prefix'])
        self.render_text_element(xml, 'key', {}, task['key'])
        self.render_text_element(xml, 'dc:comment', {}, task['comment'])

        for lang_code, lang_string, lang_field in get_languages():
            self.render_text_element(xml, 'title', {'lang': lang_code}, task['title_%s' % lang_code])
            self.render_text_element(xml, 'text', {'lang': lang_code}, task['text_%s' % lang_code])

        self.render_text_element(xml, 'start_attribute',  {'dc:uri': task['start_attribute']}, None)
        self.render_text_element(xml, 'end_attribute',  {'dc:uri': task['end_attribute']}, None)
        self.render_text_element(xml, 'days_before', {}, task['days_before'])
        self.render_text_element(xml, 'days_after', {}, task['days_after'])

        xml.startElement('conditions', {})
        if 'conditions' in task and task['conditions']:
            for condition in task['conditions']:
                self.render_text_element(xml, 'condition', {'dc:uri': condition}, None)
        xml.endElement('conditions')

        xml.startElement('catalogs', {})
        if 'catalogs' in task and task['catalogs']:
            for catalog in task['catalogs']:
                self.render_text_element(xml, 'catalog', {'dc:uri': catalog}, None)
        xml.endElement('catalogs')

        xml.endElement('task')


class TaskRenderer(TasksRenderer):

    def render_document(self, xml, tasks):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for task in tasks:
            self.render_task(xml, task)
        xml.endElement('rdmo')



from django.urls import reverse


def test_catalog_search(db, client):
    client.login(username='admin', password='admin')

    url = reverse('admin:questions_catalog_changelist') + '?q=test'
    response = client.get(url)
    assert response.status_code == 200


def test_section_search(db, client):
    client.login(username='admin', password='admin')

    url = reverse('admin:questions_section_changelist') + '?q=test'
    response = client.get(url)
    assert response.status_code == 200


def test_questionset_search(db, client):
    client.login(username='admin', password='admin')

    url = reverse('admin:questions_questionset_changelist') + '?q=test'
    response = client.get(url)
    assert response.status_code == 200


def test_question_search(db, client):
    client.login(username='admin', password='admin')

    url = reverse('admin:questions_question_changelist') + '?q=test'
    response = client.get(url)
    assert response.status_code == 200



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-26 16:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0006_auto_20160803_1619'),
    ]

    operations = [
        migrations.AddField(
            model_name='catalog',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this catalog.', null=True, verbose_name='Comment'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this catalog. The URI will be generated from this key.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this catalog (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AddField(
            model_name='catalog',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this catalog.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
        migrations.AddField(
            model_name='questionentity',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this question/questionset.', null=True, verbose_name='Comment'),
        ),
        migrations.AddField(
            model_name='questionentity',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this question/questionset. The URI will be generated from this key.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AddField(
            model_name='questionentity',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this question/questionset (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AddField(
            model_name='questionentity',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this question/questionset.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
        migrations.AddField(
            model_name='section',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this section.', null=True, verbose_name='Comment'),
        ),
        migrations.AddField(
            model_name='section',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this section. The URI will be generated from this key.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AddField(
            model_name='section',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this section (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AddField(
            model_name='section',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this section.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
        migrations.AddField(
            model_name='subsection',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this subsection.', null=True, verbose_name='Comment'),
        ),
        migrations.AddField(
            model_name='subsection',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this subsection. The URI will be generated from this key.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AddField(
            model_name='subsection',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this subsection (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AddField(
            model_name='subsection',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this subsection.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='order',
            field=models.IntegerField(default=0, help_text='The position of this catalog in lists.', verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='title_de',
            field=models.CharField(help_text='The German title for this catalog.', max_length=256, verbose_name='Title (de)'),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='title_en',
            field=models.CharField(help_text='The English title for this catalog.', max_length=256, verbose_name='Title (en)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='parent',
            field=models.ForeignKey(blank=True, help_text='The question set this question belongs to.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='questions', to='questions.QuestionEntity', verbose_name='Parent'),
        ),
        migrations.AlterField(
            model_name='question',
            name='text_de',
            field=models.TextField(help_text='The German text for this question.', verbose_name='Text (de)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='text_en',
            field=models.TextField(help_text='The English text for this question.', verbose_name='Text (en)'),
        ),
        migrations.AlterField(
            model_name='question',
            name='widget_type',
            field=models.CharField(choices=[('text', 'Text'), ('textarea', 'Textarea'), ('yesno', 'Yes/No'), ('checkbox', 'Checkboxes'), ('radio', 'Radio buttons'), ('select', 'Select drop-down'), ('range', 'Range slider'), ('date', 'Date picker')], help_text='Type of widget for this question.', max_length=12, verbose_name='Widget type'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='attribute_entity',
            field=models.ForeignKey(blank=True, help_text='The attribute/entity this question belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.AttributeEntity', verbose_name='Attribute entity'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='help_de',
            field=models.TextField(blank=True, help_text='The German help text for this question/questionset.', null=True, verbose_name='Help (de)'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='help_en',
            field=models.TextField(blank=True, help_text='The English help text for this question/questionset.', null=True, verbose_name='Help (en)'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='label_de',
            field=models.TextField(help_text='The German label for this question/questionset (auto-generated).', verbose_name='Label (de)'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='label_en',
            field=models.TextField(help_text='The English label for this question/questionset (auto-generated).', verbose_name='Label (en)'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='order',
            field=models.IntegerField(default=0, help_text='The position of this subsection in lists.', verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='subsection',
            field=models.ForeignKey(help_text='The section this question belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='entities', to='questions.Subsection', verbose_name='Catalog'),
        ),
        migrations.AlterField(
            model_name='section',
            name='catalog',
            field=models.ForeignKey(help_text='The catalog this section belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='sections', to='questions.Catalog', verbose_name='Catalog'),
        ),
        migrations.AlterField(
            model_name='section',
            name='label_de',
            field=models.TextField(help_text='The German label for this section (auto-generated).', verbose_name='Label (de)'),
        ),
        migrations.AlterField(
            model_name='section',
            name='label_en',
            field=models.TextField(help_text='The English label for this section (auto-generated).', verbose_name='Label (en)'),
        ),
        migrations.AlterField(
            model_name='section',
            name='order',
            field=models.IntegerField(default=0, help_text='The position of this section in lists.', verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='section',
            name='title_de',
            field=models.CharField(help_text='The German title for this section.', max_length=256, verbose_name='Title (de)'),
        ),
        migrations.AlterField(
            model_name='section',
            name='title_en',
            field=models.CharField(help_text='The English title for this section.', max_length=256, verbose_name='Title (en)'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='label_de',
            field=models.TextField(help_text='The German label for this subsection (auto-generated).', verbose_name='Label (de)'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='label_en',
            field=models.TextField(help_text='The English label for this subsection (auto-generated).', verbose_name='Label (en)'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='order',
            field=models.IntegerField(default=0, help_text='The position of this subsection in lists.', verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='section',
            field=models.ForeignKey(help_text='The section this subsection belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='subsections', to='questions.Section', verbose_name='Catalog'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='title_de',
            field=models.CharField(help_text='The German title for this subsection.', max_length=256, verbose_name='Title (de)'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='title_en',
            field=models.CharField(help_text='The English title for this subsection.', max_length=256, verbose_name='Title (en)'),
        ),
    ]



import pytest
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Task
from ..serializers.v1 import TaskSerializer
from ..validators import TaskLockedValidator


def test_create(db):
    TaskLockedValidator()({
        'locked': False
    })


def test_create_locked(db):
    TaskLockedValidator()({
        'locked': True
    })


def test_update(db):
    task = Task.objects.first()

    TaskLockedValidator(task)({
        'locked': False
    })


def test_update_error(db):
    task = Task.objects.first()
    task.locked = True
    task.save()

    with pytest.raises(ValidationError):
        TaskLockedValidator(task)({
            'locked': True
        })


def test_update_lock(db):
    task = Task.objects.first()

    TaskLockedValidator(task)({
        'locked': True
    })


def test_update_unlock(db):
    task = Task.objects.first()
    task.locked = True
    task.save()

    TaskLockedValidator(task)({
        'locked': False
    })


def test_serializer_create(db):
    validator = TaskLockedValidator()
    validator.set_context(TaskSerializer())

    validator({
        'locked': False
    })


def test_serializer_create_locked(db):
    validator = TaskLockedValidator()
    validator.set_context(TaskSerializer())

    validator({
        'locked': True
    })


def test_serializer_update(db):
    task = Task.objects.first()

    validator = TaskLockedValidator()
    validator.set_context(TaskSerializer(instance=task))

    validator({})


def test_serializer_update_error(db):
    task = Task.objects.first()
    task.locked = True
    task.save()

    validator = TaskLockedValidator()
    validator.set_context(TaskSerializer(instance=task))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'locked': True
        })


def test_serializer_update_lock(db):
    task = Task.objects.first()

    validator = TaskLockedValidator()
    validator.set_context(TaskSerializer(instance=task))

    validator({
        'locked': True
    })


def test_serializer_update_unlock(db):
    task = Task.objects.first()
    task.locked = True
    task.save()

    validator = TaskLockedValidator()
    validator.set_context(TaskSerializer(instance=task))

    validator({
        'locked': False
    })



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-25 15:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0025_refactoring'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='verbosename',
            options={'verbose_name': 'Verbose name', 'verbose_name_plural': 'Verbose names'},
        ),
        migrations.AddField(
            model_name='attributeentity',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this attribute/entity.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='optionsets',
            field=models.ManyToManyField(blank=True, help_text='Option sets for this attribute.', to='options.OptionSet', verbose_name='Option sets'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='unit',
            field=models.CharField(blank=True, help_text='Unit of values for this attribute.', max_length=64, null=True, verbose_name='Unit'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='value_type',
            field=models.CharField(choices=[('text', 'Text'), ('url', 'URL'), ('integer', 'Integer'), ('float', 'Float'), ('boolean', 'Boolean'), ('datetime', 'Datetime'), ('options', 'Options')], help_text='Type of values for this attribute.', max_length=8, verbose_name='Value type'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this attribute/entity.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='conditions',
            field=models.ManyToManyField(blank=True, help_text='List of conditions evaluated for this attribute/entity.', to='conditions.Condition', verbose_name='Conditions'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='is_attribute',
            field=models.BooleanField(default=False, help_text='Designates whether this attribute/entity is an attribute (auto-generated).', verbose_name='is attribute'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='is_collection',
            field=models.BooleanField(default=False, help_text='Designates whether this attribute/entity is a collection.', verbose_name='is collection'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this attribute/entity. The URI will be generated from this key.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='label',
            field=models.CharField(db_index=True, help_text='The label to be displayed this attribute/entity (auto-generated).', max_length=512, verbose_name='Label'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, help_text='Parent entity in the domain model.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='domain.AttributeEntity', verbose_name='Parent entity'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='parent_collection',
            field=models.ForeignKey(blank=True, default=None, help_text='Next collection entity upwards in the domain model (auto-generated).', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='domain.AttributeEntity', verbose_name='Parent collection'),
        ),
        migrations.AlterField(
            model_name='attributeentity',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this attribute/entity set (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AlterField(
            model_name='range',
            name='attribute',
            field=models.OneToOneField(help_text='Attribute this verbose name belongs to.', on_delete=django.db.models.deletion.CASCADE, to='domain.Attribute', verbose_name='Attribute'),
        ),
        migrations.AlterField(
            model_name='range',
            name='maximum',
            field=models.FloatField(help_text='Maximum value for this attribute.', verbose_name='Maximum'),
        ),
        migrations.AlterField(
            model_name='range',
            name='minimum',
            field=models.FloatField(help_text='Minimal value for this attribute.', verbose_name='Minimum'),
        ),
        migrations.AlterField(
            model_name='range',
            name='step',
            field=models.FloatField(help_text='Step in which this attribute can be incremented/decremented.', verbose_name='Step'),
        ),
        migrations.AlterField(
            model_name='verbosename',
            name='attribute_entity',
            field=models.OneToOneField(help_text='Attribute/entity this verbose name belongs to.', on_delete=django.db.models.deletion.CASCADE, to='domain.AttributeEntity', verbose_name='Attribute entity'),
        ),
        migrations.AlterField(
            model_name='verbosename',
            name='name_de',
            field=models.CharField(help_text='German name displayed for this attribute/entity (e.g. Projekt).', max_length=256, verbose_name='Name (de)'),
        ),
        migrations.AlterField(
            model_name='verbosename',
            name='name_en',
            field=models.CharField(help_text='English name displayed for this attribute/entity (e.g. project).', max_length=256, verbose_name='Name (en)'),
        ),
        migrations.AlterField(
            model_name='verbosename',
            name='name_plural_de',
            field=models.CharField(help_text='German plural name displayed for this attribute/entity (e.g. Projekte).', max_length=256, verbose_name='Plural name (de)'),
        ),
        migrations.AlterField(
            model_name='verbosename',
            name='name_plural_en',
            field=models.CharField(help_text='English plural name displayed for this attribute/entity (e.g. projects).', max_length=256, verbose_name='Plural name (en)'),
        ),
    ]



from django.urls import include, path
from rest_framework import routers

from ..viewsets import GroupViewSet, SettingsViewSet, SitesViewSet

router = routers.DefaultRouter()
router.register(r'settings', SettingsViewSet, basename='setting')
router.register(r'sites', SitesViewSet, basename='site')
router.register(r'groups', GroupViewSet, basename='group')

urlpatterns = [
    path('accounts/', include('rdmo.accounts.urls.v1')),
    path('conditions/', include('rdmo.conditions.urls.v1')),
    path('domain/', include('rdmo.domain.urls.v1')),
    path('options/', include('rdmo.options.urls.v1')),
    path('overlays/', include('rdmo.overlays.urls.v1')),
    path('projects/', include('rdmo.projects.urls.v1')),
    path('questions/', include('rdmo.questions.urls.v1')),
    path('tasks/', include('rdmo.tasks.urls.v1')),
    path('views/', include('rdmo.views.urls.v1')),

    path('core/', include(router.urls)),
]



from django import template
from django.utils.translation import gettext_lazy as _
from django.utils.safestring import mark_safe

from ..utils import get_full_name

register = template.Library()


@register.simple_tag()
def full_name(user):
    return get_full_name(user)


@register.simple_tag()
def user_data_as_dl(user):
    html = '<dl>'
    html += '<dt>%(key)s</dt><dd>%(value)s</dd>' % {
        'key': _('Name'),
        'value': get_full_name(user)
    }
    for additional_value in user.additional_values.all():
        html += '<dt>%(key)s</dt><dd>%(value)s</dd>' % {
            'key': additional_value.field.text,
            'value': additional_value.value
        }
    html += '</dl>'
    return mark_safe(html)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def run_data_migration(apps, schema_editor):
    QuestionEntity = apps.get_model('questions', 'QuestionEntity')
    QuestionSet = apps.get_model('questions', 'QuestionSet')
    Question = apps.get_model('questions', 'Question')
    QuestionItem = apps.get_model('questions', 'QuestionItem')

    # migrate questionsets
    for questionentity in QuestionEntity.objects.filter(question=None):
        questionset = QuestionSet(
            id=questionentity.id,
            uri=questionentity.uri,
            uri_prefix=questionentity.uri_prefix,
            key=questionentity.key,
            path=questionentity.path,
            comment=questionentity.comment,
            attribute_entity=questionentity.attribute_entity if questionentity.is_collection else None,
            subsection=questionentity.subsection,
            is_collection=questionentity.is_collection,
            order=questionentity.order,
            help_en=questionentity.help_en,
            help_de=questionentity.help_de,
            created=questionentity.created,
            updated=questionentity.updated
        )
        questionset.save()

        for condition in questionentity.conditions.all():
            questionset.conditions.add(condition)

        # migrate questions in questionsets
        for question in questionentity.questions.all():
            questionitem = QuestionItem(
                id=question.id,
                uri=question.uri,
                uri_prefix=question.uri_prefix,
                key=question.key,
                path=question.path,
                comment=question.comment,
                attribute_entity=question.attribute_entity,
                questionset=questionset,
                is_collection=question.is_collection,
                order=question.order,
                text_en=question.text_en,
                text_de=question.text_de,
                help_en=question.help_en,
                help_de=question.help_de,
                widget_type=question.widget_type,
                value_type=question.value_type,
                unit=question.unit,
                created=question.created,
                updated=question.updated
            )
            questionitem.save()

            for optionset in question.optionsets.all():
                questionitem.optionsets.add(optionset)

            for condition in question.conditions.all():
                questionitem.conditions.add(condition)


    # migrate questions without questionsets
    for question in Question.objects.filter(parent=None):
        questionset = QuestionSet(
            id=question.id,
            uri=question.uri,
            uri_prefix=question.uri_prefix,
            key=question.key,
            path=question.path,
            comment=question.comment,
            attribute_entity=None,
            subsection=question.subsection,
            is_collection=False,
            order=question.order,
            help_en='',
            help_de='',
            created=question.created,
            updated=question.updated
        )
        questionset.save()

        for condition in question.conditions.all():
            questionset.conditions.add(condition)

        questionitem = QuestionItem(
            id=question.id,
            uri=question.uri + '/' + question.key,
            uri_prefix=question.uri_prefix,
            key=question.key,
            path=question.path + '/' + question.key,
            comment=question.comment,
            attribute_entity=question.attribute_entity,
            questionset=questionset,
            is_collection=question.is_collection,
            order=0,
            text_en=question.text_en,
            text_de=question.text_de,
            help_en=question.help_en,
            help_de=question.help_de,
            widget_type=question.widget_type,
            value_type=question.value_type,
            unit=question.unit,
            created=question.created,
            updated=question.updated
        )
        questionitem.save()

        for optionset in question.optionsets.all():
            questionitem.optionsets.add(optionset)


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0025_questionset_and_questionitem'),
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-07-26 14:06
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0008_meta'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='condition',
            name='attribute_entity',
        ),
        migrations.RemoveField(
            model_name='condition',
            name='source_attribute',
        ),
        migrations.RemoveField(
            model_name='condition',
            name='target_option',
        ),
        migrations.DeleteModel(
            name='Condition',
        ),
    ]



# Generated by Django 2.2.13 on 2020-08-24 11:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0052_available'),
    ]

    operations = [
        migrations.AlterField(
            model_name='question',
            name='attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute this question belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='questions', to='domain.Attribute', verbose_name='Attribute'),
        ),
        migrations.AlterField(
            model_name='question',
            name='conditions',
            field=models.ManyToManyField(blank=True, help_text='List of conditions evaluated for this question.', related_name='questions', to='conditions.Condition', verbose_name='Conditions'),
        ),
        migrations.AlterField(
            model_name='question',
            name='optionsets',
            field=models.ManyToManyField(blank=True, help_text='Option sets for this question.', related_name='questions', to='options.OptionSet', verbose_name='Option sets'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute this questionset belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='questionsets', to='domain.Attribute', verbose_name='Attribute'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='conditions',
            field=models.ManyToManyField(blank=True, help_text='List of conditions evaluated for this questionset.', related_name='questionsets', to='conditions.Condition', verbose_name='Conditions'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-06-05 11:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0014_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='unit',
            field=models.CharField(blank=True, help_text='Unit for this question.', max_length=64, verbose_name='Unit', default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='question',
            name='value_type',
            field=models.CharField(choices=[(b'text', 'Text'), (b'url', 'URL'), (b'integer', 'Integer'), (b'float', 'Float'), (b'boolean', 'Boolean'), (b'datetime', 'Datetime'), (b'options', 'Options')], default='text', help_text='Type of value for this question.', max_length=8, verbose_name='Value type'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-28 12:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0029_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='attribute',
            options={'permissions': (('view_attribute', 'Can view Attribute'),), 'verbose_name': 'Attribute', 'verbose_name_plural': 'Attributes'},
        ),
        migrations.AlterModelOptions(
            name='attributeentity',
            options={'ordering': ('uri',), 'permissions': (('view_attributeentity', 'Can view Attribute entity'),), 'verbose_name': 'Attribute entity', 'verbose_name_plural': 'Attribute entities'},
        ),
        migrations.AlterModelOptions(
            name='range',
            options={'ordering': ('attribute',), 'permissions': (('view_range', 'Can view Range'),), 'verbose_name': 'Range', 'verbose_name_plural': 'Ranges'},
        ),
        migrations.AlterModelOptions(
            name='verbosename',
            options={'permissions': (('view_verbosename', 'Can view Verbose name'),), 'verbose_name': 'Verbose name', 'verbose_name_plural': 'Verbose names'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-07-26 15:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0002_many_to_many_for_conditions'),
        ('domain', '0009_remove_condition'),
    ]

    operations = [
        migrations.AddField(
            model_name='attributeentity',
            name='conditions',
            field=models.ManyToManyField(to='conditions.Condition'),
        ),
    ]



import logging

from django.conf import settings
from django.contrib.auth import logout
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.shortcuts import render

from rdmo.core.utils import get_next, get_referer_path_info

from .forms import ProfileForm, RemoveForm
from .utils import delete_user

log = logging.getLogger(__name__)


@login_required()
def profile_update(request):
    if settings.PROFILE_UPDATE:
        log.debug('Update user %s', request.user.username)

        form = ProfileForm(request.POST or None, instance=request.user)

        if request.method == 'POST':
            if 'cancel' in request.POST:
                log.debug('User %s uldate cancelled', request.user.username)
                return HttpResponseRedirect(get_next(request))

            if form.is_valid():
                form.save()
                return HttpResponseRedirect(get_next(request))

        return render(request, 'profile/profile_update_form.html', {
            'form': form,
            'next': get_referer_path_info(request, default='/')
        })
    else:
        return render(request, 'profile/profile_update_closed.html')


@login_required()
def remove_user(request):
    if settings.PROFILE_DELETE:
        log.debug('Remove user form for "%s"', request.user.username)

        form = RemoveForm(request.POST or None, request=request)

        if request.method == 'POST':
            if 'cancel' in request.POST:
                log.debug('User %s removal cancelled', str(request.user))

                if settings.PROFILE_UPDATE:
                    return HttpResponseRedirect('/account')
                else:
                    return HttpResponseRedirect('/')

            if form.is_valid():
                log.debug('Deleting user %s', request.user.username)

                if delete_user(request.user, request.POST['email'], request.POST['password']):
                    logout(request)
                    return render(request, 'profile/profile_remove_success.html')
                else:
                    return render(request, 'profile/profile_remove_failed.html')

        return render(request, 'profile/profile_remove_form.html', {
            'form': form,
            'next': get_referer_path_info(request, default='/')
        })
    else:
        return render(request, 'profile/profile_remove_closed.html')


def terms_of_use(request):
    return render(request, 'account/terms_of_use.html')



from datetime import date, timedelta
from itertools import zip_longest

from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from rdmo.tasks.models import Task

from ..managers import IssueManager


class Issue(models.Model):

    objects = IssueManager()

    ISSUE_STATUS_OPEN = 'open'
    ISSUE_STATUS_IN_PROGRESS = 'in_progress'
    ISSUE_STATUS_CLOSED = 'closed'
    ISSUE_STATUS_CHOICES = (
        (ISSUE_STATUS_OPEN, _('open')),
        (ISSUE_STATUS_IN_PROGRESS, _('in progress')),
        (ISSUE_STATUS_CLOSED, _('closed'))
    )

    project = models.ForeignKey(
        'Project', on_delete=models.CASCADE, related_name='issues',
        verbose_name=_('Project'),
        help_text=_('The project for this issue.')
    )
    task = models.ForeignKey(
        Task, on_delete=models.CASCADE, related_name='issues',
        verbose_name=_('Task'),
        help_text=_('The task for this issue.')
    )
    status = models.CharField(
        max_length=12, choices=ISSUE_STATUS_CHOICES, default=ISSUE_STATUS_OPEN,
        verbose_name=_('Status'),
        help_text=_('The status for this issue.')
    )

    class Meta:
        ordering = ('project__title', )
        verbose_name = _('Issue')
        verbose_name_plural = _('Issues')

    def __str__(self):
        return '%s / %s / %s' % (self.project.title, self.task, self.status)

    def get_absolute_url(self):
        return reverse('project', kwargs={'pk': self.project.pk})

    def resolve(self, values):
        for condition in self.task.conditions.all():
            if condition.resolve(values):
                return True

    @property
    def dates(self):
        values = self.project.values.filter(snapshot=None)

        if self.task.start_attribute:
            start_values = values.filter(attribute=self.task.start_attribute)
        else:
            start_values = []

        if self.task.end_attribute:
            end_values = values.filter(attribute=self.task.end_attribute)
        else:
            end_values = []

        days_before = timedelta(self.task.days_before) if self.task.days_before else timedelta()
        days_after = timedelta(self.task.days_after) if self.task.days_after else timedelta()

        dates = []
        for start_value, end_value in zip_longest(start_values, end_values):

            if start_value and start_value.value and isinstance(start_value.value, date):
                start_date = start_value.value
            else:
                start_date = None

            if end_value and end_value.value and isinstance(end_value.value, date):
                end_date = end_value.value
            else:
                end_date = None

            if start_date and end_date:
                dates.append((start_date - days_before, end_date + days_after))
            elif start_date:
                dates.append((start_date - days_before + days_after, ))
            elif end_date:
                dates.append((end_date - days_before + days_after, ))

        return dates


class IssueResource(models.Model):

    issue = models.ForeignKey(
        'Issue', on_delete=models.CASCADE, related_name='resources',
        verbose_name=_('Issue'),
        help_text=_('The issue for this issue resource.')
    )
    integration = models.ForeignKey(
        'Integration', on_delete=models.CASCADE, related_name='resources',
        verbose_name=_('Integration'),
        help_text=_('The integration for this issue resource.')
    )
    url = models.URLField(
        verbose_name=_('URL'),
        help_text=_('The URL of this issue resource.')
    )

    class Meta:
        ordering = ('issue__project__title', 'issue')
        verbose_name = _('Issue resource')
        verbose_name_plural = _('Issue resources')

    def __str__(self):
        return '%s / %s / %s' % (self.issue.project.title, self.issue, self.url)



from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.viewsets import ReadOnlyModelViewSet

from rdmo.core.permissions import HasModelPermission, HasObjectPermission

from .serializers.v1 import UserSerializer
from .utils import is_site_manager


class UserViewSetMixin(object):

    def get_users_for_user(self, user):
        if user.is_authenticated:
            if user.has_perm('auth.view_user'):
                return get_user_model().objects.all()
            elif is_site_manager(user):
                current_site = Site.objects.get_current()
                return get_user_model().objects.filter(role__member=current_site)
        return get_user_model().objects.none()


class UserViewSet(UserViewSetMixin, ReadOnlyModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = UserSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'username',
        'first_name',
        'last_name',
        'email',
        'groups',
    )

    def get_queryset(self):
        return self.get_users_for_user(self.request.user) \
                   .prefetch_related('groups', 'role__member', 'role__manager', 'memberships')



# Generated by Django 2.2.16 on 2021-01-22 19:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0021_related_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='condition',
            name='locked',
            field=models.BooleanField(default=False, help_text='Designates whether this condition can be changed.', verbose_name='Locked'),
        ),
    ]



from django import forms
from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.core.validators import EmailValidator
from django.db.models import Q
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from rdmo.core.constants import VALUE_TYPE_FILE
from rdmo.core.plugins import get_plugin
from rdmo.core.utils import markdown2html

from .constants import ROLE_CHOICES
from .models import (Integration, IntegrationOption, Invite, Membership,
                     Project, Snapshot)


class CatalogChoiceField(forms.ModelChoiceField):

    def label_from_instance(self, obj):
        return mark_safe('<b>%s</b></br>%s' % (obj.title, markdown2html(obj.help)))


class TasksMultipleChoiceField(forms.ModelMultipleChoiceField):

    def label_from_instance(self, obj):
        return mark_safe('<b>%s</b></br>%s' % (obj.title, markdown2html(obj.text)))


class ViewsMultipleChoiceField(forms.ModelMultipleChoiceField):

    def label_from_instance(self, obj):
        return mark_safe('<b>%s</b></br>%s' % (obj.title, markdown2html(obj.help)))


class ProjectForm(forms.ModelForm):

    use_required_attribute = False

    def __init__(self, *args, **kwargs):
        catalogs = kwargs.pop('catalogs')
        projects = kwargs.pop('projects')
        super().__init__(*args, **kwargs)
        self.fields['title'].widget.attrs.update({
            'autofocus': True
        })
        self.fields['catalog'].queryset = catalogs
        self.fields['catalog'].empty_label = None
        self.fields['catalog'].initial = catalogs.first()

        if settings.NESTED_PROJECTS:
            self.fields['parent'].queryset = projects

    class Meta:
        model = Project

        fields = ['title', 'description', 'catalog']
        if settings.NESTED_PROJECTS:
            fields += ['parent']

        field_classes = {
            'catalog': CatalogChoiceField
        }
        widgets = {
            'catalog': forms.RadioSelect()
        }


class ProjectUpdateInformationForm(forms.ModelForm):

    use_required_attribute = False

    class Meta:
        model = Project
        fields = ('title', 'description')


class ProjectUpdateCatalogForm(forms.ModelForm):

    use_required_attribute = False

    def __init__(self, *args, **kwargs):
        catalogs = kwargs.pop('catalogs')
        super().__init__(*args, **kwargs)
        self.fields['catalog'].queryset = catalogs
        self.fields['catalog'].empty_label = None

    class Meta:
        model = Project
        fields = ('catalog', )
        field_classes = {
            'catalog': CatalogChoiceField
        }
        widgets = {
            'catalog': forms.RadioSelect()
        }


class ProjectUpdateTasksForm(forms.ModelForm):

    use_required_attribute = False

    def __init__(self, *args, **kwargs):
        tasks = kwargs.pop('tasks')
        super().__init__(*args, **kwargs)
        self.fields['tasks'].queryset = tasks

    class Meta:
        model = Project
        fields = ('tasks', )
        field_classes = {
            'tasks': TasksMultipleChoiceField
        }
        widgets = {
            'tasks': forms.CheckboxSelectMultiple()
        }


class ProjectUpdateViewsForm(forms.ModelForm):

    use_required_attribute = False

    def __init__(self, *args, **kwargs):
        views = kwargs.pop('views')
        super().__init__(*args, **kwargs)
        self.fields['views'].queryset = views

    class Meta:
        model = Project
        fields = ('views', )
        field_classes = {
            'views': ViewsMultipleChoiceField
        }
        widgets = {
            'views': forms.CheckboxSelectMultiple()
        }


class ProjectUpdateParentForm(forms.ModelForm):

    use_required_attribute = False

    def __init__(self, *args, **kwargs):
        projects = kwargs.pop('projects')
        super().__init__(*args, **kwargs)
        self.fields['parent'].queryset = projects

    class Meta:
        model = Project
        fields = ('parent', )


class SnapshotCreateForm(forms.ModelForm):

    use_required_attribute = False

    class Meta:
        model = Snapshot
        fields = ('title', 'description')

    def __init__(self, *args, **kwargs):
        self.project = kwargs.pop('project')
        super(SnapshotCreateForm, self).__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        self.instance.project = self.project
        return super(SnapshotCreateForm, self).save(*args, **kwargs)


class MembershipCreateForm(forms.Form):

    use_required_attribute = False

    username_or_email = forms.CharField(widget=forms.TextInput(attrs={'placeholder': _('Username or email')}),
                                        label=_('User'),
                                        help_text=_('The username or email of the new user.'))
    role = forms.CharField(widget=forms.RadioSelect(choices=ROLE_CHOICES),
                           initial='author')

    def __init__(self, *args, **kwargs):
        self.project = kwargs.pop('project')
        self.is_site_manager = kwargs.pop('is_site_manager')
        super().__init__(*args, **kwargs)

        if self.is_site_manager:
            self.fields['silent'] = forms.BooleanField(
                required=False,
                label=_('Add member silently'),
                help_text=_('As site manager or admin, you can directly add users without notifying them via email, when you check the following checkbox.')
            )

    def clean_username_or_email(self):
        username_or_email = self.cleaned_data['username_or_email']

        # check if it is a registered
        try:
            self.cleaned_data['user'] = get_user_model().objects.get(Q(username=username_or_email) | Q(email__iexact=username_or_email))
            self.cleaned_data['email'] = self.cleaned_data['user'].email

            if self.cleaned_data['user'] in self.project.user.all():
                raise ValidationError(_('The user is already a member of the project.'))

        except get_user_model().DoesNotExist:
            if settings.PROJECT_SEND_INVITE:
                # check if it is a valid email address, this will raise the correct ValidationError
                EmailValidator()(username_or_email)

                self.cleaned_data['user'] = None
                self.cleaned_data['email'] = username_or_email
            else:
                self.cleaned_data['user'] = None
                self.cleaned_data['email'] = None
                raise ValidationError(_('A user with this username or email was not found. Only registered users can be invited.'))

    def clean(self):
        if self.cleaned_data.get('silent') is True and self.cleaned_data.get('user') is None:
            raise ValidationError(_('Only existing users can be added silently.'))

    def save(self):
        if self.is_site_manager and self.cleaned_data.get('silent') is True:
            Membership.objects.create(
                project=self.project,
                user=self.cleaned_data.get('user'),
                role=self.cleaned_data.get('role')
            )
        else:
            invite, created = Invite.objects.get_or_create(
                project=self.project,
                user=self.cleaned_data.get('user'),
                email=self.cleaned_data.get('email')
            )
            invite.role = self.cleaned_data.get('role')
            invite.make_token()
            invite.save()

            return invite


class IntegrationForm(forms.ModelForm):

    class Meta:
        model = Integration
        fields = ()

    def __init__(self, *args, **kwargs):
        self.project = kwargs.pop('project')
        self.provider_key = kwargs.pop('provider_key', None)
        super().__init__(*args, **kwargs)

        # get the provider
        if self.provider_key:
            self.provider = get_plugin('SERVICE_PROVIDERS', self.provider_key)
        else:
            self.provider = self.instance.provider

        # add fields for the integration options
        for field in self.provider.fields:
            try:
                initial = IntegrationOption.objects.get(integration=self.instance, key=field.get('key')).value
            except IntegrationOption.DoesNotExist:
                initial = None

            if field.get('placeholder'):
                attrs = {'placeholder': field.get('placeholder')}
            self.fields[field.get('key')] = forms.CharField(widget=forms.TextInput(attrs=attrs),
                                                            initial=initial, required=field.get('required', True))

    def save(self):
        # the the project and the provider_key
        self.instance.project = self.project
        if self.provider_key:
            self.instance.provider_key = self.provider_key

        # call the form's save method
        super().save()

        # save the integration options
        self.instance.save_options(self.cleaned_data)


class IssueSendForm(forms.Form):

    class AttachmentViewsField(forms.ModelMultipleChoiceField):
        def label_from_instance(self, obj):
            return _('Attach %s') % obj.title

    class AttachmentFilesField(forms.ModelMultipleChoiceField):
        def label_from_instance(self, obj):
            return _('Attach %s') % obj.file_name

    class AttachmentSnapshotField(forms.ModelChoiceField):
        def label_from_instance(self, obj):
            return obj.title

    subject = forms.CharField(label=_('Subject'), max_length=128)
    message = forms.CharField(label=_('Message'), widget=forms.Textarea)

    def __init__(self, *args, **kwargs):
        self.project = kwargs.pop('project')
        super().__init__(*args, **kwargs)

        self.fields['attachments_answers'] = forms.MultipleChoiceField(
            label=_('Answers'), widget=forms.CheckboxSelectMultiple, required=False,
            choices=[('project_answers', _('Attach the output of "View answers".'))]
        )
        self.fields['attachments_views'] = self.AttachmentViewsField(
            label=_('Views'), widget=forms.CheckboxSelectMultiple, required=False,
            queryset=self.project.views.all(), to_field_name='id'
        )
        self.fields['attachments_files'] = self.AttachmentFilesField(
            label=_('Files'), widget=forms.CheckboxSelectMultiple, required=False,
            queryset=self.project.values.filter(snapshot=None)
                                        .filter(value_type=VALUE_TYPE_FILE)
                                        .order_by('file'),
            to_field_name='id'
        )
        self.fields['attachments_snapshot'] = self.AttachmentSnapshotField(
            label=_('Snapshot'), widget=forms.RadioSelect, required=False,
            queryset=self.project.snapshots.all(), empty_label=_('Current')
        )
        self.fields['attachments_format'] = forms.ChoiceField(
            label=_('Format'), widget=forms.RadioSelect, required=False,
            choices=settings.EXPORT_FORMATS
        )

    def clean(self):
        cleaned_data = super().clean()

        if cleaned_data.get('attachments_answers') or cleaned_data.get('attachments_views'):
            if not cleaned_data.get('attachments_format'):
                self.add_error('attachments_format', _('This field is required.'))


class IssueMailForm(forms.Form):

    if settings.EMAIL_RECIPIENTS_CHOICES:
        recipients = forms.MultipleChoiceField(label=_('Recipients'), widget=forms.CheckboxSelectMultiple,
                                               required=not settings.EMAIL_RECIPIENTS_INPUT,
                                               choices=settings.EMAIL_RECIPIENTS_CHOICES)

    if settings.EMAIL_RECIPIENTS_INPUT:
        recipients_input = forms.CharField(label=_('Recipients'), widget=forms.Textarea(attrs={
            'placeholder': _('Enter recipients line by line')
        }), required=not settings.EMAIL_RECIPIENTS_CHOICES)

    def clean(self):
        cleaned_data = super().clean()

        if settings.EMAIL_RECIPIENTS_INPUT and \
                cleaned_data.get('recipients') == [] and \
                cleaned_data.get('recipients_input') == []:
            self.add_error('recipients_input', _('This field is required.'))

    def clean_recipients_input(self):
        email_validator = EmailValidator()
        cleaned_data = []

        for line in self.cleaned_data['recipients_input'].splitlines():
            email = line.strip()
            email_validator(email)
            cleaned_data.append(email)

        return cleaned_data


class UploadFileForm(forms.Form):
    uploaded_file = forms.FileField(
        label='Select a file',
    )



import logging
import re

import defusedxml.ElementTree as ET

log = logging.getLogger(__name__)


def read_xml_file(file_name):
    try:
        return ET.parse(file_name).getroot()
    except Exception as e:
        log.error('Xml parsing error: ' + str(e))


def parse_xml_string(string):
    try:
        return ET.fromstring(string)
    except Exception as e:
        log.error('Xml parsing error: ' + str(e))


def flat_xml_to_elements(treenode):
    elements = []
    ns_map = get_ns_map(treenode)
    uri_attrib = get_ns_tag('dc:uri', ns_map)

    for node in treenode:

        element = {
            'uri': get_uri(node, ns_map),
            'type': node.tag
        }

        for subnode in node:
            tag = strip_ns(subnode.tag, ns_map)

            if uri_attrib in subnode.attrib:
                # this node has an uri!
                element[tag] = subnode.attrib[uri_attrib]
            elif 'lang' in subnode.attrib:
                # this node has the lang attribute!
                element['%s_%s' % (tag, subnode.attrib['lang'])] = subnode.text
            elif list(subnode):
                # this node is a list!
                element[tag] = [subsubnode.attrib[uri_attrib] for subsubnode in subnode]
            else:
                element[tag] = subnode.text

        elements.append(element)

    elements = sort_elements_by_key(elements, 'uri')
    return elements


def get_ns_tag(tag, ns_map):
    tag_split = tag.split(':')
    try:
        return '{%s}%s' % (ns_map[tag_split[0]], tag_split[1])
    except KeyError:
        return None


def get_ns_map(treenode):
    ns_map = {}
    treestring = ET.tostring(treenode, encoding='utf8', method='xml')

    for match in re.finditer(r'(xmlns:)(.*?)(=")(.*?)(")', str(treestring)):
        if match:
            ns_map[match.group(2)] = match.group(4)

    return ns_map


def get_uri(treenode, ns_map):
    ns_tag = get_ns_tag('dc:uri', ns_map)
    if ns_tag is not None:
        return treenode.attrib.get(ns_tag)


def strip_ns(tag, ns_map):
    for ns in ns_map.values():
        if tag.startswith('{%s}' % ns):
            return tag.replace('{%s}' % ns, '')
    return tag


def filter_elements_by_type(elements, element_type):
    for element in elements:
        if element['type'] == element_type:
            yield element


def sort_elements_by_key(dictlist, key, reverse=False):
    return sorted(dictlist, key=lambda k: k[key], reverse=reverse)



from django.contrib import admin

from .models import Overlay


class OverlayAdmin(admin.ModelAdmin):
    search_fields = ('user__username', 'url_name', 'current')
    list_display = ('user', 'site', 'url_name', 'current')
    list_filter = ('url_name', 'current')


admin.site.register(Overlay, OverlayAdmin)



import re

from django.conf import settings
from django.core import mail
from django.urls import reverse

from django.contrib.auth import get_user_model

users = (
    ('editor', 'editor'),
    ('reviewer', 'reviewer'),
    ('user', 'user'),
    ('api', 'api'),
    ('anonymous', None),
)


def test_get_profile_update(db, client):
    """
    An authorized GET request to the profile update form returns the form.
    """
    client.login(username='user', password='user')

    url = reverse('profile_update')
    response = client.get(url)
    assert response.status_code == 200


def test_get_profile_update_redirect(db, client):
    """
    An unauthorized GET request to the profile update form gets
    redirected to login.
    """
    url = reverse('profile_update')
    response = client.get(url)
    assert response.status_code == 302
    assert response.url == reverse('account_login') + '?next=' + url


def test_post_profile_update(db, client):
    """
    An authorized POST request to the profile update form updates the
    user and redirects to home.
    """
    client.login(username='user', password='user')

    url = reverse('profile_update')
    data = {
        'email': 'test@example.com',
        'first_name': 'Albert',
        'last_name': 'Admin',
        'text': 'text',
        'textarea': 'textarea'
    }
    response = client.post(url, data)

    if settings.PROFILE_UPDATE:
        assert response.status_code == 302
        assert response.url == reverse('home')
    else:
        assert response.status_code == 200


def test_post_profile_update_cancel(db, client):
    """
    An authorized POST request to the profile update form updates with
    cancel redirects to home.
    """
    client.login(username='user', password='user')

    url = reverse('profile_update')
    data = {
        'email': 'test@example.com',
        'first_name': 'Albert',
        'last_name': 'Admin',
        'cancel': 'cancel'
    }
    response = client.post(url, data)

    if settings.PROFILE_UPDATE:
        assert response.status_code == 302
        assert response.url == reverse('home')
    else:
        assert response.status_code == 200


def test_post_profile_update_cancel2(db, client):
    """
    An authorized POST request to the profile update form updates with
    cancel and the next field redirects to the given url.
    """
    client.login(username='user', password='user')

    url = reverse('profile_update')
    data = {
        'email': 'test@example.com',
        'first_name': 'Albert',
        'last_name': 'Admin',
        'cancel': 'cancel',
        'next': reverse('projects')
    }
    response = client.post(url, data)

    if settings.PROFILE_UPDATE:
        assert response.status_code == 302
        assert response.url == reverse('projects')
    else:
        assert response.status_code == 200


def test_post_profile_update_next(db, client):
    """
    An authorized POST request to the profile update form with next field
    updates the user and redirects to the given url.
    """
    client.login(username='user', password='user')

    url = reverse('profile_update')
    data = {
        'email': 'test@example.com',
        'first_name': 'Albert',
        'last_name': 'Admin',
        'text': 'text',
        'textarea': 'textarea',
        'next': reverse('projects')
    }
    response = client.post(url, data)

    if settings.PROFILE_UPDATE:
        assert response.status_code == 302
        assert response.url == reverse('projects')
    else:
        assert response.status_code == 200


def test_post_profile_update_next2(db, client):
    """
    An authorized POST request to the profile update form with next
    field set to profile_update updates the user and redirects to home.
    """
    client.login(username='user', password='user')

    url = reverse('profile_update')
    data = {
        'email': 'test@example.com',
        'first_name': 'Albert',
        'last_name': 'Admin',
        'text': 'text',
        'textarea': 'textarea',
        'next': reverse('profile_update')
    }
    response = client.post(url, data)

    if settings.PROFILE_UPDATE:
        assert response.status_code == 302
        assert response.url == reverse('home')
    else:
        assert response.status_code == 200


def test_password_change_get(db, client):
    """
    An authorized GET request to the password change form returns the form.
    """
    if settings.ACCOUNT:
        client.login(username='user', password='user')

        url = reverse('account_change_password')
        response = client.get(url)
        assert response.status_code == 200


def test_password_change_post(db, client):
    """
    An authorized POST request to the password change form updates the
    password and redirects to home.
    """
    if settings.ACCOUNT:
        client.login(username='user', password='user')

        url = reverse('account_change_password')
        data = {
            'old_password': 'user',
            'new_password1': 'resu',
            'new_password2': 'resu',
        }
        response = client.post(url, data)
        assert response.status_code == 200


def test_password_reset_get(db, client):
    """
    A GET request to the password reset form returns the form.
    """
    if settings.ACCOUNT:
        url = reverse('account_reset_password')
        response = client.get(url)
        assert response.status_code == 200


def test_password_reset_post_invalid(db, client):
    """
    A POST request to the password reset form with an invalid mail address
    sends no mail.
    """
    if settings.ACCOUNT:
        url = reverse('account_reset_password')
        data = {'email': 'wrong@example.com'}
        response = client.post(url, data)
        assert response.status_code == 200
        assert len(mail.outbox) == 0


def test_password_reset_post_valid(db, client):
    """
    A POST request to the password reset form with an invalid mail address
    sends a mail with a correct link.
    """
    if settings.ACCOUNT:
        url = reverse('account_reset_password')
        data = {'email': 'user@example.com'}
        response = client.post(url, data)
        assert response.status_code == 302
        assert response.url == reverse('account_reset_password_done')
        assert len(mail.outbox) == 1

        # get the link from the mail
        urls = re.findall(r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', mail.outbox[0].body)
        assert len(urls) == 1

        # get the password_reset page
        response = client.get(urls[0])
        assert response.status_code == 302
        assert response.url == reverse('account_reset_password_from_key', args=['4','set-password'])


def test_remove_user_get(db, client):
    if settings.PROFILE_DELETE:
        client.login(username='user', password='user')

        url = reverse('profile_remove')
        response = client.get(url)
        assert response.status_code == 200


def test_remove_user_post(db, client):
    if settings.PROFILE_DELETE:
        client.login(username='user', password='user')

        url = reverse('profile_remove')
        data = {
            'email': 'user@example.com',
            'password': 'user',
            'consent': True
        }
        response = client.post(url, data)
        assert response.status_code == 200
        assert not get_user_model().objects.filter(username='user').exists()


def test_remove_user_post_invalid_email(db, client):
    if settings.PROFILE_DELETE:
        client.login(username='user', password='user')

        url = reverse('profile_remove')
        data = {
            'email': 'invalid',
            'password': 'user',
            'consent': True
        }
        response = client.post(url, data)
        assert response.status_code == 200
        assert get_user_model().objects.filter(username='user').exists()


def test_remove_user_post_invalid_password(db, client):
    if settings.PROFILE_DELETE:
        client.login(username='user', password='user')

        url = reverse('profile_remove')
        data = {
            'email': 'user@example.com',
            'password': 'invalid',
            'consent': True
        }
        response = client.post(url, data)
        assert response.status_code == 200
        assert get_user_model().objects.filter(username='user').exists()


def test_remove_user_post_invalid_consent(db, client):
    if settings.PROFILE_DELETE:
        client.login(username='user', password='user')

        url = reverse('profile_remove')
        data = {
            'email': 'user@example.com',
            'password': 'user',
            'consent': False
        }

        response = client.post(url, data)
        assert response.status_code == 200
        assert get_user_model().objects.filter(username='user').exists()


def test_signup(db, client):
    url = reverse('account_signup')
    response = client.post(url, {
        'email': 'test@example.com',
        'username': 'test',
        'first_name': 'test',
        'last_name': 'test',
        'password1': 'test',
        'password2': 'test',
    })

    assert response.status_code == 302
    assert response.url == '/'


def test_signup_next(db, client):
    url = reverse('account_signup') + '?next=/about/'
    response = client.post(url, {
        'email': 'test@example.com',
        'username': 'test',
        'first_name': 'test',
        'last_name': 'test',
        'password1': 'test',
        'password2': 'test',
    })

    assert response.status_code == 302
    assert response.url == '/about/'



import pytest

from ..models import Integration, Issue, Membership, Project, Snapshot, Value

projects = [1, 2, 3, 4, 5]


def test_integration_str(db):
    instances = Integration.objects.all()
    for instance in instances:
        assert str(instance)


def test_issue_str(db):
    instances = Issue.objects.all()
    for instance in instances:
        assert str(instance)


def test_project_str(db):
    instances = Project.objects.all()
    for instance in instances:
        assert str(instance)


def test_membership_str(db):
    instances = Membership.objects.all()
    for instance in instances:
        assert str(instance)


def test_snapshot_str(db):
    instances = Snapshot.objects.all()
    for instance in instances:
        assert str(instance)


def test_value_str(db):
    instances = Value.objects.all()
    for instance in instances:
        assert str(instance)


@pytest.mark.parametrize('project_id', projects)
def test_project_delete(db, project_id):
    project = Project.objects.get(id=project_id)
    project_parent_id = project.parent_id if project.parent else None
    project_children = [child.id for child in project.get_children()]

    project.delete()

    for child_id in project_children:
        child = Project.objects.get(id=child_id)

        if project_parent_id is None:
            assert child.parent is None
        else:
            assert child.parent.id is project_parent_id



# Generated by Django 2.2.18 on 2021-04-20 13:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0026_optionset_option_locked'),
        ('questions', '0057_question_default_text'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='default_option',
            field=models.ForeignKey(blank=True, help_text='The default option for this question. To be used with regular optionsets.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='options.Option', verbose_name='Default option'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-30 12:37
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0005_data_migration'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='task',
            options={'ordering': ('uri',), 'verbose_name': 'Task', 'verbose_name_plural': 'Tasks'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-13 10:58
from __future__ import unicode_literals

from django.db import migrations

def set_null_to_blank(queryset, fields):
    for element in queryset:
        for field in fields:
            setattr(element, field, getattr(element, field) or '')
        element.save()


def run_data_migration(apps, schema_editor):
    Catalog = apps.get_model('questions', 'Catalog')
    Section = apps.get_model('questions', 'Section')
    QuestionSet = apps.get_model('questions', 'QuestionSet')
    Question = apps.get_model('questions', 'Question')

    set_null_to_blank(Catalog.objects.all(), [
        'uri',
        'uri_prefix',
        'key',
        'comment',
        'title_lang1',
        'title_lang2',
        'title_lang3',
        'title_lang4',
        'title_lang5',
    ])
    set_null_to_blank(Section.objects.all(), [
        'uri',
        'uri_prefix',
        'key',
        'path',
        'comment',
        'title_lang1',
        'title_lang2',
        'title_lang3',
        'title_lang4',
        'title_lang5',
    ])
    set_null_to_blank(QuestionSet.objects.all(), [
        'uri',
        'uri_prefix',
        'key',
        'path',
        'comment',
        'title_lang1',
        'title_lang2',
        'title_lang3',
        'title_lang4',
        'title_lang5',
        'help_lang1',
        'help_lang2',
        'help_lang3',
        'help_lang4',
        'help_lang5',
    ])
    set_null_to_blank(Question.objects.all(), [
        'uri',
        'uri_prefix',
        'key',
        'comment',
        'text_lang1',
        'text_lang2',
        'text_lang3',
        'text_lang4',
        'text_lang5',
        'help_lang1',
        'help_lang2',
        'help_lang3',
        'help_lang4',
        'help_lang5',
        'value_type',
        'widget_type',
        'unit',
    ])


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0040_add_language_fields'),
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



from django.urls import include, path
from rest_framework_extensions.routers import ExtendedDefaultRouter

from ..viewsets import (IntegrationViewSet, IssueViewSet, MembershipViewSet,
                        ProjectIntegrationViewSet, ProjectIssueViewSet,
                        ProjectMembershipViewSet, ProjectQuestionSetViewSet,
                        ProjectSnapshotViewSet, ProjectValueViewSet,
                        ProjectViewSet, SnapshotViewSet, ValueViewSet)

app_name = 'v1-projects'

router = ExtendedDefaultRouter()
project_route = router.register(r'projects', ProjectViewSet, basename='project')
project_route.register(r'memberships', ProjectMembershipViewSet, basename='project-membership',
                       parents_query_lookups=['project'])
project_route.register(r'integrations', ProjectIntegrationViewSet, basename='project-integration',
                       parents_query_lookups=['project'])
project_route.register(r'issues', ProjectIssueViewSet, basename='project-issue',
                       parents_query_lookups=['project'])
project_route.register(r'snapshots', ProjectSnapshotViewSet, basename='project-snapshot',
                       parents_query_lookups=['project'])
project_route.register(r'values', ProjectValueViewSet, basename='project-value',
                       parents_query_lookups=['project'])
project_route.register(r'questionsets', ProjectQuestionSetViewSet, basename='project-questionset',
                       parents_query_lookups=['project'])
router.register(r'memberships', MembershipViewSet, basename='membership')
router.register(r'integrations', IntegrationViewSet, basename='integration')
router.register(r'issues', IssueViewSet, basename='issue')
router.register(r'snapshots', SnapshotViewSet, basename='snapshot')
router.register(r'values', ValueViewSet, basename='value')


urlpatterns = [
    path('', include(router.urls)),
]



# Generated by Django 2.2rc1 on 2019-03-26 13:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0022_remove_null_true'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='task',
            options={'ordering': ('uri',), 'verbose_name': 'Task', 'verbose_name_plural': 'Tasks'},
        ),
    ]



from rest_framework import serializers
from rest_framework.reverse import reverse

from rdmo.conditions.models import Condition
from rdmo.core.serializers import TranslationSerializerMixin
from rdmo.core.utils import get_language_warning
from rdmo.questions.models import QuestionSet

from ..models import Option, OptionSet
from ..validators import (OptionLockedValidator, OptionSetLockedValidator,
                          OptionSetUniqueURIValidator,
                          OptionUniqueURIValidator)


class QuestionSerializer(serializers.ModelSerializer):

    class Meta:
        model = QuestionSet
        fields = (
            'id',
            'uri'
        )


class ConditionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Condition
        fields = (
            'id',
            'uri'
        )


class OptionSetSerializer(serializers.ModelSerializer):

    key = serializers.SlugField(required=True)
    questions = QuestionSerializer(many=True, read_only=True)

    class Meta:
        model = OptionSet
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'order',
            'provider_key',
            'conditions',
            'questions'
        )
        validators = (
            OptionSetUniqueURIValidator(),
            OptionSetLockedValidator()
        )


class OptionSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    key = serializers.SlugField(required=True)
    optionset = serializers.PrimaryKeyRelatedField(queryset=OptionSet.objects.all(), required=True)
    conditions = ConditionSerializer(many=True, read_only=True)
    values_count = serializers.IntegerField(read_only=True)
    projects_count = serializers.IntegerField(read_only=True)

    class Meta:
        model = Option
        fields = (
            'id',
            'optionset',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'order',
            'text',
            'label',
            'additional_input',
            'conditions',
            'values_count',
            'projects_count'
        )
        trans_fields = (
            'text',
        )
        validators = (
            OptionUniqueURIValidator(),
            OptionLockedValidator()
        )


class OptionSetIndexOptionSerializer(serializers.ModelSerializer):

    class Meta:
        model = Option
        fields = (
            'id',
            'uri'
        )


class OptionSetIndexSerializer(serializers.ModelSerializer):

    options = OptionSetIndexOptionSerializer(many=True)

    class Meta:
        model = OptionSet
        fields = (
            'id',
            'uri',
            'options'
        )


class OptionIndexSerializer(serializers.ModelSerializer):

    class Meta:
        model = Option
        fields = (
            'id',
            'optionset',
            'uri',
            'text'
        )


class ConditionNestedSerializer(serializers.ModelSerializer):

    class Meta:
        model = Condition
        fields = (
            'id',
            'uri'
        )


class ProviderNestedSerializer(serializers.Serializer):

    key = serializers.CharField()
    label = serializers.CharField()
    class_name = serializers.CharField()

    class Meta:
        fields = (
            'key',
            'label',
            'class_name'
        )


class OptionNestedSerializer(serializers.ModelSerializer):

    warning = serializers.SerializerMethodField()
    xml_url = serializers.SerializerMethodField()

    class Meta:
        model = Option
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'path',
            'locked',
            'order',
            'text',
            'warning',
            'xml_url'
        )

    def get_warning(self, obj):
        return get_language_warning(obj, 'text')

    def get_xml_url(self, obj):
        return reverse('v1-options:option-detail-export', args=[obj.pk])


class OptionSetNestedSerializer(serializers.ModelSerializer):

    options = OptionNestedSerializer(many=True)
    conditions = ConditionNestedSerializer(many=True)
    provider = ProviderNestedSerializer()
    xml_url = serializers.SerializerMethodField()

    class Meta:
        model = OptionSet
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'order',
            'locked',
            'provider',
            'options',
            'conditions',
            'xml_url'
        )

    def get_xml_url(self, obj):
        return reverse('v1-options:optionset-detail-export', args=[obj.pk])



import base64

from rest_framework import serializers

from ..models import Project, Snapshot, Value


class ValueSerializer(serializers.ModelSerializer):

    attribute = serializers.CharField(source='attribute.uri', default=None, read_only=True)
    option = serializers.CharField(source='option.uri', default=None, read_only=True)
    file_content = serializers.SerializerMethodField()

    class Meta:
        model = Value
        fields = (
            'attribute',
            'set_prefix',
            'set_index',
            'collection_index',
            'text',
            'option',
            'file_name',
            'file_content',
            'value_type',
            'unit',
            'external_id',
            'created',
            'updated'
        )

    def get_file_content(self, obj):
        if obj.file:
            return base64.b64encode(obj.file.read())


class SnapshotSerializer(serializers.ModelSerializer):

    values = serializers.SerializerMethodField()

    class Meta:
        model = Snapshot
        fields = (
            'title',
            'description',
            'values',
            'created',
            'updated'
        )

    def get_values(self, obj):
        values = Value.objects.filter(snapshot=obj)
        serializer = ValueSerializer(instance=values, many=True)
        return serializer.data


class ProjectSerializer(serializers.ModelSerializer):

    snapshots = SnapshotSerializer(many=True)
    values = serializers.SerializerMethodField()

    catalog = serializers.CharField(source='catalog.uri', default=None, read_only=True)
    tasks = serializers.SerializerMethodField()
    views = serializers.SerializerMethodField()

    class Meta:
        model = Project
        fields = (
            'title',
            'description',
            'catalog',
            'tasks',
            'views',
            'snapshots',
            'values',
            'created',
            'updated'
        )

    def get_values(self, obj):
        values = Value.objects.filter(project=obj, snapshot=None)
        serializer = ValueSerializer(instance=values, many=True)
        return serializer.data

    def get_tasks(self, obj):
        return [task.uri for task in obj.tasks.all()]

    def get_views(self, obj):
        return [view.uri for view in obj.views.all()]



import pytest
from django.urls import reverse

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('api', 'api'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_questionset_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'author': [1, 3, 5],
    'guest': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

urlnames = {
    'list': 'v1-projects:project-questionset-list',
    'detail': 'v1-projects:project-questionset-detail'
}

projects = [1, 2, 3, 4, 5]
questionsets = [1]


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('questionset_id', questionsets)
def test_detail(db, client, username, password, project_id, questionset_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[project_id, questionset_id])
    response = client.get(url)

    if project_id in view_questionset_permission_map.get(username, []):
        assert response.status_code == 200
        assert response.json().get('id') == questionset_id
    else:
        assert response.status_code == 404



from django.urls import path
from django.views.generic.base import RedirectView

from ..views import ImportView, UploadView

urlpatterns = [
    path('', RedirectView.as_view(pattern_name='catalogs'), name='management'),
    path('upload/', UploadView.as_view(), name='upload'),
    path('import/', ImportView.as_view(), name='import')
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-01-29 16:18
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0016_remove_timeframe'),
    ]

    operations = [
        migrations.RenameField(
            model_name='task',
            old_name='text_en',
            new_name='text_lang1',
        ),
        migrations.RenameField(
            model_name='task',
            old_name='title_en',
            new_name='title_lang1',
        ),
    ]



import xml.etree.ElementTree as et

import pytest
from django.urls import reverse

users = (
    ('editor', 'editor'),
    ('reviewer', 'reviewer'),
    ('user', 'user'),
    ('api', 'api'),
    ('anonymous', None),
)

status_map = {
    'views': {
        'editor': 200, 'reviewer': 200, 'api': 200, 'user': 403, 'anonymous': 302
    },
    'views_export': {
        'editor': 200, 'reviewer': 200, 'api': 200, 'user': 403, 'anonymous': 302
    }
}

export_formats = ('xml', 'rtf', 'odt', 'docx', 'html', 'markdown', 'tex', 'pdf')


@pytest.mark.parametrize('username,password', users)
def test_views(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('views')
    response = client.get(url)
    assert response.status_code == status_map['views'][username]


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('export_format', export_formats)
def test_views_export(db, client, username, password, export_format):
    client.login(username=username, password=password)

    url = reverse('views_export', args=[export_format])
    response = client.get(url)
    assert response.status_code == status_map['views_export'][username]

    if response.status_code == 200:
        if export_format == 'xml':
            root = et.fromstring(response.content)
            assert root.tag == 'rdmo'
            for child in root:
                assert child.tag in ['view']



from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.sites.models import Site
from rest_framework import status, viewsets
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from rdmo.core.permissions import HasModelPermission

from .serializers import GroupSerializer, SiteSerializer


class SettingsViewSet(viewsets.GenericViewSet):

    permission_classes = (IsAuthenticated, )

    def list(self, request, *args, **kwargs):
        return Response({
            key.lower(): getattr(settings, key) for key in settings.SETTINGS_API
        })


class SitesViewSet(viewsets.ReadOnlyModelViewSet):
    permission_classes = (HasModelPermission, )
    queryset = Site.objects.all()
    serializer_class = SiteSerializer


class GroupViewSet(viewsets.ReadOnlyModelViewSet):
    permission_classes = (HasModelPermission, )
    queryset = Group.objects.all()
    serializer_class = GroupSerializer


class CopyModelMixin:

    @action(detail=True, methods=['PUT'], permission_classes=[HasModelPermission])
    def copy(self, request, pk=None):
        # get the instance to be copied
        instance = self.get_object()

        # get the copy relevant data from the request
        uri_prefix = request.data.get('uri_prefix')
        key = request.data.get('key')

        # get the parent fields from the model
        try:
            parent_fields = instance.parent_fields
            parent_ids = [request.data.get(parent_field) for parent_field in parent_fields]
        except AttributeError:
            parent_fields = parent_ids = []

        # get the original and the original_serializer
        original = self.get_object()
        original_serializer = self.get_serializer(original)

        # merge the original_serializer with the data from the request and validate
        data = original_serializer.data
        data.update({
            'uri_prefix': uri_prefix,
            'key': key
        })
        for parent_field, parent_id in zip(parent_fields, parent_ids):
            data[parent_field] = parent_id
        validation_serializer = self.get_serializer(data=data)
        validation_serializer.is_valid(raise_exception=True)

        # perform the copy on the database
        parents = []
        for parent_field, parent_id in zip(parent_fields, parent_ids):
            parent_model = instance._meta.get_field(parent_field).remote_field.model
            parent = parent_model.objects.filter(pk=parent_id).first()
            parents.append(parent)
        instance.copy(uri_prefix, key, *parents)

        # the rest is similar to CreateModelMixin.create()
        serializer = self.get_serializer(instance)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)



from django.conf import settings
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from rdmo.core.constants import VALUE_TYPE_FILE
from rdmo.core.utils import human2bytes


class ValueValidator(object):

    requires_context = True

    def __call__(self, data, serializer):
        if data.get('value_type') == VALUE_TYPE_FILE:
            try:
                serializer.context['view'].get_object()
            except AssertionError:
                project = serializer.context['view'].project

                if project.file_size > human2bytes(settings.PROJECT_FILE_QUOTA):
                    raise serializers.ValidationError({
                        'value': [_('You reached the file quota for this project.')]
                    })



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-05 14:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0010_meta'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='text_de',
            field=models.TextField(help_text='The German text for this task.', verbose_name='Text (de)'),
        ),
        migrations.AlterField(
            model_name='task',
            name='text_en',
            field=models.TextField(help_text='The English text for this task.', verbose_name='Text (en)'),
        ),
    ]



from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model


class Command(BaseCommand):

    def handle(self, *args, **options):
        get_user_model().objects.create_superuser('admin', 'admin@example.com', 'admin')



# Generated by Django 2.2.18 on 2021-03-29 10:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0055_catalog_locked'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='is_optional',
            field=models.BooleanField(default=False, help_text='Designates whether this question is optional.', verbose_name='is optional'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-21 13:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0015_move_attribute_to_attributeentity'),
        ('projects', '0022_move_attribute_to_attributeentity'),
        ('tasks', '0012_move_attribute_to_attributeentity'),
        ('domain', '0036_remove_range_and_verbosename')
    ]

    operations = [
        # add a dummy field so that the table does not become empty after the next step
        # otherwise MySQL and SQLite will complain
        # https://code.djangoproject.com/ticket/24424
        migrations.AddField(
            model_name='attribute',
            name='_',
            field=models.BooleanField(default=False),
        ),
        migrations.RemoveField(
            model_name='attribute',
            name='attributeentity_ptr',
        ),
        migrations.DeleteModel(
            name='Attribute',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-06 15:02
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0033_remove_attribute_optionsets'),
        ('questions', '0020_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='attributeentity',
            name='conditions',
        ),
    ]
